@@ -20,7 +20,7 @@ set_config(
     html_logo=os.path.join("_images", "gamma_sklearndf_logo.png"),
     intersphinx_mapping={
         "lightgbm": ("https://lightgbm.readthedocs.io/en/latest/", None),
-        "pytools": ("https://bcg-gamma.github.io/pytools/", None),
+        "pytools": ("https://bcg-x-official.github.io/pytools/", None),
         "sklearn": ("https://scikit-learn.org/stable", None),
         "xgboost": ("https://xgboost.readthedocs.io/en/latest/", None),
     },
