@@ -1,12 +1,14 @@
 """
 Core implementation of :mod:`sklearndf.classification.extra`
 """
+
 import logging
 
 from sklearn.base import ClassifierMixin
 
 from pytools.api import AllTracker
 
+from ..._util import remove_invalid_lgbm_type_hint
 from ...wrapper import ClassifierWrapperDF, MissingEstimator
 
 log = logging.getLogger(__name__)
@@ -16,6 +18,8 @@ __all__ = ["LGBMClassifierDF", "XGBClassifierDF"]
 try:
     # import lightgbm classes only if installed
     from lightgbm.sklearn import LGBMClassifier
+
+    remove_invalid_lgbm_type_hint(LGBMClassifier)
 except ImportError:
 
     class LGBMClassifier(  # type: ignore
@@ -49,17 +53,11 @@ __tracker = AllTracker(globals())
 #
 
 
-class LGBMClassifierDF(
-    ClassifierWrapperDF[LGBMClassifier],
-    native=LGBMClassifier,
-):
+class LGBMClassifierDF(ClassifierWrapperDF[LGBMClassifier], native=LGBMClassifier):
     """Stub for DF wrapper of class ``LGBMClassifierDF``"""
 
 
-class XGBClassifierDF(
-    ClassifierWrapperDF[XGBClassifier],
-    native=XGBClassifier,
-):
+class XGBClassifierDF(ClassifierWrapperDF[XGBClassifier], native=XGBClassifier):
     """Stub for DF wrapper of class ``XGBClassifierDF``"""
 
 
