@@ -4,7 +4,7 @@ GAMMA custom two-step pipelines
 
 import logging
 from abc import ABCMeta, abstractmethod
-from typing import Any, Generic, List, Optional, TypeVar, Union
+from typing import Any, Generic, Optional, TypeVar, Union
 
 import numpy.typing as npt
 import pandas as pd
@@ -107,39 +107,10 @@ class _EstimatorPipelineDF(EstimatorDF, Generic[T_FinalEstimatorDF], metaclass=A
         """
         pass
 
-    @property
-    def feature_names_out_(self) -> pd.Index:
-        """
-        Pandas column index of all features resulting from the preprocessing step.
-
-        Same as :attr:`.feature_names_in_` if the preprocessing step is ``None``.
-        """
-        if self.preprocessing is not None:
-            return self.preprocessing.feature_names_out_
-        else:
-            return self.feature_names_in_.rename(TransformerDF.COL_FEATURE_OUT)
-
-    @property
-    def feature_names_original_(self) -> pd.Series:
-        """
-        Pandas series mapping the names of all features resulting from the preprocessing
-        step to the names of the input features they were derived from.
-
-        Returns an identity mapping of :attr:`.feature_names_in_` onto itself
-        if the preprocessing step is ``None``.
-        """
-        if self.preprocessing is not None:
-            return self.preprocessing.feature_names_original_
-        else:
-            feature_names_in_ = self.feature_names_in_
-            return feature_names_in_.to_series(index=feature_names_in_).rename_axis(
-                index=TransformerDF.COL_FEATURE_OUT
-            )
-
     # noinspection PyPep8Naming
     def fit(
         self: T_EstimatorPipelineDF,
-        X: pd.DataFrame,
+        X: Union[pd.DataFrame, pd.Series],
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         *,
         sample_weight: Optional[pd.Series] = None,
@@ -175,24 +146,44 @@ class _EstimatorPipelineDF(EstimatorDF, Generic[T_FinalEstimatorDF], metaclass=A
             self.preprocessing is None or self.preprocessing.is_fitted
         ) and self.final_estimator.is_fitted
 
+    # noinspection PyPep8Naming
+    def preprocess(self, X: pd.DataFrame) -> pd.DataFrame:
+        """
+        Preprocess the given feature values using this pipeline's preprocessing step.
+
+        If the pipeline has no preprocessing step, return the input unchanged.
+
+        :param X: input data frame with observations as rows and features as columns
+        :return: the preprocessed input data frame
+        """
+        if self.preprocessing is not None:
+            return self.preprocessing.transform(X)
+        else:
+            return X
+
     def _get_features_in(self) -> pd.Index:
         if self.preprocessing is not None:
             return self.preprocessing.feature_names_in_
         else:
             return self.final_estimator.feature_names_in_
 
+    def _get_n_features_in(self) -> int:
+        if self.preprocessing is not None:
+            return self.preprocessing.n_features_in_
+        else:
+            return self.final_estimator.n_features_in_
+
+    def _get_outputs(self) -> Optional[list[str]]:
+        if self.preprocessing is not None:
+            return self.preprocessing._get_outputs()
+        else:
+            return self.final_estimator._get_outputs()
+
     def _get_n_outputs(self) -> int:
         if self.preprocessing is not None:
-            return self.preprocessing.n_outputs_
+            return self.preprocessing._get_n_outputs()
         else:
-            return self.final_estimator.n_outputs_
-
-    # noinspection PyPep8Naming
-    def _pre_transform(self, X: pd.DataFrame) -> pd.DataFrame:
-        if self.preprocessing is not None:
-            return self.preprocessing.transform(X)
-        else:
-            return X
+            return self.final_estimator._get_n_outputs()
 
     # noinspection PyPep8Naming
     def _pre_fit_transform(
@@ -218,10 +209,10 @@ class LearnerPipelineDF(
 
     # noinspection PyPep8Naming
     def predict(
-        self, X: pd.DataFrame, **predict_params: Any
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
-        return self.final_estimator.predict(self._pre_transform(X), **predict_params)
+        return self.final_estimator.predict(self.preprocess(X), **predict_params)
 
 
 @inheritdoc(match="[see superclass]")
@@ -239,16 +230,16 @@ class SupervisedLearnerPipelineDF(
     # noinspection PyPep8Naming
     def score(
         self,
-        X: pd.DataFrame,
+        X: Union[pd.Series, pd.DataFrame],
         y: Optional[pd.Series] = None,
         sample_weight: Optional[Any] = None,
     ) -> float:
         """[see superclass]"""
         if sample_weight is None:
-            return self.final_estimator.score(self._pre_transform(X), y)
+            return self.final_estimator.score(self.preprocess(X), y)
         else:
             return self.final_estimator.score(
-                self._pre_transform(X), y, sample_weight=sample_weight
+                self.preprocess(X), y, sample_weight=sample_weight
             )
 
 
@@ -336,35 +327,29 @@ class ClassifierPipelineDF(
         """[see superclass]"""
         return "classifier"
 
-    @property
-    def classes_(self) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
-        """[see superclass]"""
-        return self.final_estimator.classes_
+    def _get_classes(self) -> Union[npt.NDArray[Any], list[npt.NDArray[Any]]]:
+        return self.final_estimator._get_classes()
 
     # noinspection PyPep8Naming
     def predict_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
+    ) -> Union[pd.DataFrame, list[pd.DataFrame]]:
         """[see superclass]"""
-        return self.classifier.predict_proba(self._pre_transform(X), **predict_params)
+        return self.classifier.predict_proba(self.preprocess(X), **predict_params)
 
     # noinspection PyPep8Naming
     def predict_log_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
+    ) -> Union[pd.DataFrame, list[pd.DataFrame]]:
         """[see superclass]"""
-        return self.classifier.predict_log_proba(
-            self._pre_transform(X), **predict_params
-        )
+        return self.classifier.predict_log_proba(self.preprocess(X), **predict_params)
 
     # noinspection PyPep8Naming
     def decision_function(
-        self, X: pd.DataFrame, **predict_params: Any
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
-        return self.classifier.decision_function(
-            self._pre_transform(X), **predict_params
-        )
+        return self.classifier.decision_function(self.preprocess(X), **predict_params)
 
 
 @inheritdoc(match="[see superclass]")
@@ -409,21 +394,19 @@ class ClusterPipelineDF(
         return "cluster"
 
     @property
-    def labels_(self) -> pd.Series:
+    def _get_labels(self) -> pd.Series:
         """[see superclass]"""
-        return self.final_estimator.labels_
+        return self.final_estimator._get_labels()
 
     # noinspection PyPep8Naming
     def fit_predict(
         self,
-        X: pd.DataFrame,
+        X: Union[pd.Series, pd.DataFrame],
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         **fit_predict_params: Any,
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
-        return self.clusterer.fit_predict(
-            self._pre_transform(X), y, **fit_predict_params
-        )
+        return self.clusterer.fit_predict(self.preprocess(X), y, **fit_predict_params)
 
 
 __tracker.validate()
