@@ -4,25 +4,36 @@ Core implementation of :mod:`sklearndf.pipeline.wrapper`
 
 import logging
 from abc import ABCMeta
-from typing import Any, Dict, Iterator, List, Sequence, Tuple, Union, cast
+from collections.abc import Iterator, Sequence
+from typing import Any, Union, cast
 
 import numpy.typing as npt
 import pandas as pd
 from pandas.core.arrays import ExtensionArray
+from scipy import sparse
 from sklearn.pipeline import FeatureUnion, Pipeline
+from sklearn.preprocessing import FunctionTransformer
 
 from pytools.api import AllTracker
 
-from sklearndf import EstimatorDF, TransformerDF
+from ..._util import hstack_frames
+from sklearndf import EstimatorDF, TransformerDF, __sklearn_1_6__, __sklearn_version__
 from sklearndf.wrapper import (
     ClassifierWrapperDF,
     RegressorWrapperDF,
     TransformerWrapperDF,
 )
 
+if __sklearn_version__ >= __sklearn_1_6__:
+    from sklearn.utils import Tags
+
 log = logging.getLogger(__name__)
 
-__all__ = ["PipelineWrapperDF", "FeatureUnionWrapperDF"]
+__all__ = [
+    "FeatureUnionSparseFrames",
+    "FeatureUnionWrapperDF",
+    "PipelineWrapperDF",
+]
 
 
 #
@@ -86,13 +97,13 @@ class PipelineWrapperDF(
             )
 
     @property
-    def steps(self) -> List[Tuple[str, EstimatorDF]]:
+    def steps(self) -> list[tuple[str, EstimatorDF]]:
         """
         The ``steps`` attribute of the underlying :class:`~sklearn.pipeline.Pipeline`.
 
         List of (name, transformer) tuples (transformers implement fit/transform).
         """
-        return cast(List[Tuple[str, EstimatorDF]], self.native_estimator.steps)
+        return cast(list[tuple[str, EstimatorDF]], self.native_estimator.steps)
 
     def __len__(self) -> int:
         """The number of steps of the pipeline."""
@@ -131,14 +142,14 @@ class PipelineWrapperDF(
         # in the pipeline
         return estimator is None or estimator == PipelineWrapperDF.PASSTHROUGH
 
-    def _transformer_steps(self) -> Iterator[Tuple[str, TransformerDF]]:
-        # make an iterator of all transform steps, i.e. excluding the final step
+    def _transformer_steps(self) -> Iterator[tuple[str, TransformerDF]]:
+        # make an iterator of all transform steps, i.e., excluding the final step
         # in case it is not a transformer
         # excludes steps whose transformer is ``None`` or ``"passthrough"``
 
         def _iter_not_none(
-            transformer_steps: Sequence[Tuple[str, EstimatorDF]]
-        ) -> Iterator[Tuple[str, TransformerDF]]:
+            transformer_steps: Sequence[tuple[str, EstimatorDF]],
+        ) -> Iterator[tuple[str, TransformerDF]]:
             return (
                 (name, cast(TransformerDF, transformer))
                 for name, transformer in transformer_steps
@@ -206,23 +217,56 @@ class PipelineWrapperDF(
         # noinspection PyProtectedMember
         return cast(str, self.native_estimator._estimator_type)
 
-    def _more_tags(self) -> Dict[str, Any]:
+    if __sklearn_version__ >= __sklearn_1_6__:
+
+        def __sklearn_tags__(self) -> Tags:
+            # forward this method call to the native estimator to ensure correct tags
+            return self.native_estimator.__sklearn_tags__()
+
+    def _more_tags(self) -> dict[str, Any]:
         return cast(
-            Dict[str, Any], getattr(self.native_estimator, "_more_tags", lambda: {})()
+            dict[str, Any], getattr(self.native_estimator, "_more_tags", lambda: {})()
         )
 
 
-class FeatureUnionWrapperDF(TransformerWrapperDF[FeatureUnion], metaclass=ABCMeta):
+class FeatureUnionSparseFrames(
+    FeatureUnion,  # type:ignore
+):
+    """
+    FeatureUnion transformer that returns sparse data frames instead of arrays if one or
+    more of its transformers return a sparse data frame.
+    """
+
+    # noinspection PyPep8Naming
+    def _hstack(
+        self, Xs: list[Union[npt.NDArray[Any], sparse.spmatrix, pd.DataFrame]]
+    ) -> Union[npt.NDArray[Any], sparse.spmatrix, pd.DataFrame]:
+        if all(isinstance(X, pd.DataFrame) for X in Xs):
+            return hstack_frames(
+                Xs, prefixes=[name for name, _ in self.transformer_list]
+            )
+        else:
+            return super()._hstack(Xs)
+
+
+class FeatureUnionWrapperDF(
+    TransformerWrapperDF[FeatureUnionSparseFrames], metaclass=ABCMeta
+):
     """
     DF wrapper for `scikit-learn` class :class:`~sklearn.pipeline.FeatureUnion`.
     """
 
+    DROP = "drop"
+    PASSTHROUGH = "passthrough"
+
+    __native_base_class__ = FeatureUnionSparseFrames
+
     @staticmethod
     def _prepend_features_out(features_out: pd.Index, name_prefix: str) -> pd.Index:
         return pd.Index(data=f"{name_prefix}__" + features_out.astype(str))
 
     def _get_features_original(self) -> pd.Series:
-        # concatenate output->input mappings from all included transformers other than
+        # concatenate output-to-input mappings from all included transformers other than
         # ones stated as ``None`` or ``"drop"`` or any other string
 
         # prepend the name of the transformer so the resulting feature name is
@@ -256,18 +300,29 @@ class FeatureUnionWrapperDF(TransformerWrapperDF[FeatureUnion], metaclass=ABCMet
         # prepend the name of the transformer so the resulting feature name is
         # `<name>__<output column of sub-transformer>
 
-        # noinspection PyProtectedMember
+        name: str
+        transformer: Union[TransformerDF, str, FunctionTransformer]
+
         indices = [
             self._prepend_features_out(
-                features_out=transformer.feature_names_out_, name_prefix=name
+                features_out=(
+                    self._get_features_in()
+                    if (
+                        isinstance(transformer, FunctionTransformer)
+                        and transformer.func is None
+                    )
+                    else cast(TransformerDF, transformer).feature_names_out_
+                ),
+                name_prefix=name,
             )
-            for name, transformer, _ in self.native_estimator._iter()
+            for name, transformer in self.native_estimator.transformer_list
+            if transformer != FeatureUnionWrapperDF.DROP
         ]
 
         if len(indices) == 0:
             return pd.Index()
         else:
-            return indices[0].append(other=indices[1:])
+            return indices[0].append(indices[1:])
 
 
 #
