@@ -1,21 +1,30 @@
 """
 Core implementation of :mod:`sklearndf.regression` loaded
-from sklearn 0.22 onwards
+from sklearn 0.23 onwards
 """
+
 import logging
 
-from sklearn.ensemble import StackingRegressor
+from sklearn.linear_model import GammaRegressor, PoissonRegressor, TweedieRegressor
 
 from pytools.api import AllTracker
 
-from ..wrapper.stacking import StackingRegressorWrapperDF
+from ..wrapper import RegressorWrapperDF
 
 log = logging.getLogger(__name__)
 
-__all__ = ["StackingRegressorDF"]
+__all__ = [
+    "GammaRegressorDF",
+    "PoissonRegressorDF",
+    "TweedieRegressorDF",
+]
 
 __imported_estimators = {name for name in globals().keys() if name.endswith("DF")}
 
+#
+# type variables
+#
+
 
 #
 # Ensure all symbols introduced below are included in __all__
@@ -29,11 +38,16 @@ __tracker = AllTracker(globals())
 #
 
 
-class StackingRegressorDF(
-    StackingRegressorWrapperDF[StackingRegressor],
-    native=StackingRegressor,
-):
-    """Stub for DF wrapper of class ``StackingRegressor``"""
+class PoissonRegressorDF(RegressorWrapperDF[PoissonRegressor], native=PoissonRegressor):
+    """Stub for DF wrapper of class ``PoissonRegressor``"""
+
+
+class GammaRegressorDF(RegressorWrapperDF[GammaRegressor], native=GammaRegressor):
+    """Stub for DF wrapper of class ``GammaRegressor``"""
+
+
+class TweedieRegressorDF(RegressorWrapperDF[TweedieRegressor], native=TweedieRegressor):
+    """Stub for DF wrapper of class ``TweedieRegressor``"""
 
 
 #
@@ -47,7 +61,6 @@ __tracker.validate()
 # validate that __all__ comprises all symbols ending in "DF", and no others
 #
 
-
 __estimators = {
     sym
     for sym in dir()
