@@ -1,12 +1,14 @@
 """
 Core implementation of :mod:`sklearndf.regression.extra`
 """
+
 import logging
 
 from sklearn.base import RegressorMixin
 
 from pytools.api import AllTracker
 
+from ..._util import remove_invalid_lgbm_type_hint
 from ...wrapper import MissingEstimator, RegressorWrapperDF
 
 log = logging.getLogger(__name__)
@@ -17,6 +19,10 @@ try:
     # import lightgbm classes only if installed
     from lightgbm.sklearn import LGBMRegressor
 
+    lgbm_type = LGBMRegressor
+
+    remove_invalid_lgbm_type_hint(lgbm_type)
+
 except ImportError:
 
     class LGBMRegressor(  # type: ignore
@@ -51,17 +57,11 @@ __tracker = AllTracker(globals())
 #
 
 
-class LGBMRegressorDF(
-    RegressorWrapperDF[LGBMRegressor],
-    native=LGBMRegressor,
-):
+class LGBMRegressorDF(RegressorWrapperDF[LGBMRegressor], native=LGBMRegressor):
     """Stub for DF wrapper of class ``LGBMRegressorDF``"""
 
 
-class XGBRegressorDF(
-    RegressorWrapperDF[XGBRegressor],
-    native=XGBRegressor,
-):
+class XGBRegressorDF(RegressorWrapperDF[XGBRegressor], native=XGBRegressor):
     """Stub for DF wrapper of class ``XGBRegressorDF``"""
 
 
