@@ -17,6 +17,7 @@ from pytools.api import AllTracker
 from ...transformation.wrapper import (
     ColumnPreservingTransformerWrapperDF,
     NumpyTransformerWrapperDF,
+    SingleColumnTransformerWrapperDF,
 )
 from ...wrapper import MetaEstimatorWrapperDF, RegressorWrapperDF
 
@@ -81,7 +82,7 @@ class PartialFitRegressorWrapperDF(
     # noinspection PyPep8Naming
     def partial_fit(
         self: T_PartialFitRegressorWrapperDF,
-        X: pd.DataFrame,
+        X: Union[pd.Series, pd.DataFrame],
         y: Union[pd.Series, pd.DataFrame],
         sample_weight: Optional[pd.Series] = None,
     ) -> T_PartialFitRegressorWrapperDF:
@@ -97,7 +98,7 @@ class PartialFitRegressorWrapperDF(
         :param sample_weight: optional weights applied to individual samples
         :return: ``self``
         """
-        self._check_parameter_types(X, y)
+        X, y = self._validate_parameter_types(X, y)
         self._partial_fit(X, y, sample_weight=sample_weight)
 
         return self
@@ -149,6 +150,7 @@ class RegressorTransformerWrapperDF(
 
 class IsotonicRegressionWrapperDF(
     RegressorTransformerWrapperDF[IsotonicRegression],
+    SingleColumnTransformerWrapperDF[IsotonicRegression],
     NumpyTransformerWrapperDF[IsotonicRegression],
     metaclass=ABCMeta,
 ):
@@ -156,23 +158,10 @@ class IsotonicRegressionWrapperDF(
     DF wrapper for :class:`sklearn.isotonic.IsotonicRegression`.
     """
 
-    # noinspection PyPep8Naming
-    def _check_parameter_types(
-        self,
-        X: pd.DataFrame,
-        y: Optional[pd.Series],
-        *,
-        expected_columns: pd.Index = None,
-    ) -> None:
-        super()._check_parameter_types(X, y, expected_columns=expected_columns)
-        if X.shape[1] != 1:
-            raise ValueError(
-                f"arg X expected to have exactly 1 column but has {X.shape[1]} columns"
-            )
-
     # noinspection PyPep8Naming
     def _adjust_X_type_for_delegate(self, X: pd.DataFrame) -> npt.NDArray[Any]:
-        return super()._adjust_X_type_for_delegate(X).ravel()
+        arr = super()._adjust_X_type_for_delegate(X)
+        return arr.ravel()
 
 
 #
