@@ -4,7 +4,7 @@ Core implementation of :mod:`sklearndf.transformation`
 
 import logging
 
-from sklearn.compose import ColumnTransformer
+from sklearn.base import TransformerMixin
 from sklearn.cross_decomposition import PLSSVD
 from sklearn.decomposition import (
     NMF,
@@ -22,7 +22,12 @@ from sklearn.decomposition import (
     TruncatedSVD,
 )
 from sklearn.feature_extraction import DictVectorizer, FeatureHasher
-from sklearn.feature_extraction.text import HashingVectorizer, TfidfTransformer
+from sklearn.feature_extraction.text import (
+    CountVectorizer,
+    HashingVectorizer,
+    TfidfTransformer,
+    TfidfVectorizer,
+)
 from sklearn.feature_selection import (
     RFE,
     RFECV,
@@ -74,6 +79,7 @@ from pytools.api import AllTracker
 from .wrapper import (
     AdditiveChi2SamplerWrapperDF,
     ColumnPreservingTransformerWrapperDF,
+    ColumnTransformerSparseFrames,
     ColumnTransformerWrapperDF,
     ComponentsDimensionalityReductionWrapperDF,
     FeatureSelectionWrapperDF,
@@ -84,8 +90,12 @@ from .wrapper import (
     NComponentsDimensionalityReductionWrapperDF,
     OneHotEncoderWrapperDF,
     PolynomialTransformerWrapperDF,
+    VectorizerWrapperDF,
 )
 
+# noinspection PyProtectedMember
+from .wrapper._wrapper import PCAWrapperDF
+
 log = logging.getLogger(__name__)
 
 __all__ = [
@@ -93,6 +103,8 @@ __all__ = [
     "BernoulliRBMDF",
     "BinarizerDF",
     "ColumnTransformerDF",
+    "CountVectorizerDF",
+    "CountVectorizerTx",
     "DictVectorizerDF",
     "DictionaryLearningDF",
     "FactorAnalysisDF",
@@ -146,10 +158,13 @@ __all__ = [
     "SparseRandomProjectionDF",
     "StandardScalerDF",
     "TfidfTransformerDF",
+    "TfidfVectorizerDF",
+    "TfidfVectorizerTx",
     "TruncatedSVDDF",
     "VarianceThresholdDF",
 ]
 
+
 __imported_estimators = {name for name in globals().keys() if name.endswith("DF")}
 
 
@@ -170,8 +185,7 @@ __tracker = AllTracker(globals())
 
 
 class ColumnTransformerDF(
-    ColumnTransformerWrapperDF,
-    native=ColumnTransformer,
+    ColumnTransformerWrapperDF, native=ColumnTransformerSparseFrames
 ):
     """Stub for DF wrapper of class ``ColumnTransformer``"""
 
@@ -185,30 +199,59 @@ class PLSSVDDF(ColumnPreservingTransformerWrapperDF[PLSSVD], native=PLSSVD):
     """Stub for DF wrapper of class ``PLSSVD``"""
 
 
+#
+# feature_extraction
+#
+
+
 class FeatureHasherDF(
-    ColumnPreservingTransformerWrapperDF[FeatureHasher],
-    native=FeatureHasher,
+    ColumnPreservingTransformerWrapperDF[FeatureHasher], native=FeatureHasher
 ):
     """Stub for DF wrapper of class ``FeatureHasher``"""
 
 
-class DictVectorizerDF(
-    ColumnPreservingTransformerWrapperDF[DictVectorizer],
-    native=DictVectorizer,
-):
+class DictVectorizerDF(VectorizerWrapperDF[DictVectorizer], native=DictVectorizer):
     """Stub for DF wrapper of class ``DictVectorizer``"""
 
 
 class HashingVectorizerDF(
-    ColumnPreservingTransformerWrapperDF[HashingVectorizer],
-    native=HashingVectorizer,
+    VectorizerWrapperDF[HashingVectorizer], native=HashingVectorizer
 ):
     """Stub for DF wrapper of class ``HashingVectorizer``"""
 
 
+class CountVectorizerTx(
+    CountVectorizer,  # type: ignore
+    TransformerMixin,  # type: ignore
+):
+    """
+    Subclass of ``CountVectorizer`` that makes it a transformer.
+    """
+
+
+class CountVectorizerDF(
+    VectorizerWrapperDF[CountVectorizerTx], native=CountVectorizerTx
+):
+    """Stub for DF wrapper of class ``CountVectorizer``"""
+
+
+class TfidfVectorizerTx(
+    TfidfVectorizer,  # type: ignore
+    TransformerMixin,  # type: ignore
+):
+    """
+    Subclass of ``CountVectorizer`` that makes it a transformer.
+    """
+
+
+class TfidfVectorizerDF(
+    VectorizerWrapperDF[TfidfVectorizerTx], native=TfidfVectorizerTx
+):
+    """Stub for DF wrapper of class ``TfidfVectorizer``"""
+
+
 class TfidfTransformerDF(
-    ColumnPreservingTransformerWrapperDF[TfidfTransformer],
-    native=TfidfTransformer,
+    ColumnPreservingTransformerWrapperDF[TfidfTransformer], native=TfidfTransformer
 ):
     """Stub for DF wrapper of class ``TfidfTransformer``"""
 
@@ -224,31 +267,19 @@ class SimpleImputerDF(ImputerWrapperDF[SimpleImputer], native=SimpleImputer):
     """Stub for DF wrapper of class ``SimpleImputer``"""
 
 
-class MissingIndicatorDF(
-    MissingIndicatorWrapperDF,
-    native=MissingIndicator,
-):
+class MissingIndicatorDF(MissingIndicatorWrapperDF, native=MissingIndicator):
     """Stub for DF wrapper of class ``MissingIndicator``"""
 
 
-class IterativeImputerDF(
-    ImputerWrapperDF[IterativeImputer],
-    native=IterativeImputer,
-):
+class IterativeImputerDF(ImputerWrapperDF[IterativeImputer], native=IterativeImputer):
     """Stub for DF wrapper of class ``IterativeImputer``"""
 
 
-class IsomapDF(
-    IsomapWrapperDF,
-    native=Isomap,
-):
+class IsomapDF(IsomapWrapperDF, native=Isomap):
     """Stub for DF wrapper of class ``Isomap``"""
 
 
-class AdditiveChi2SamplerDF(
-    AdditiveChi2SamplerWrapperDF,
-    native=AdditiveChi2Sampler,
-):
+class AdditiveChi2SamplerDF(AdditiveChi2SamplerWrapperDF, native=AdditiveChi2Sampler):
     """Stub for DF wrapper of class ``AdditiveChi2Sampler``"""
 
 
@@ -270,57 +301,45 @@ class NeighborhoodComponentsAnalysisDF(
 
 
 class MinMaxScalerDF(
-    ColumnPreservingTransformerWrapperDF[MinMaxScaler],
-    native=MinMaxScaler,
+    ColumnPreservingTransformerWrapperDF[MinMaxScaler], native=MinMaxScaler
 ):
     """Stub for DF wrapper of class ``MinMaxScaler``"""
 
 
 class StandardScalerDF(
-    ColumnPreservingTransformerWrapperDF[StandardScaler],
-    native=StandardScaler,
+    ColumnPreservingTransformerWrapperDF[StandardScaler], native=StandardScaler
 ):
     """Stub for DF wrapper of class ``StandardScaler``"""
 
 
 class MaxAbsScalerDF(
-    ColumnPreservingTransformerWrapperDF[MaxAbsScaler],
-    native=MaxAbsScaler,
+    ColumnPreservingTransformerWrapperDF[MaxAbsScaler], native=MaxAbsScaler
 ):
     """Stub for DF wrapper of class ``MaxAbsScaler``"""
 
 
 class RobustScalerDF(
-    ColumnPreservingTransformerWrapperDF[RobustScaler],
-    native=RobustScaler,
+    ColumnPreservingTransformerWrapperDF[RobustScaler], native=RobustScaler
 ):
     """Stub for DF wrapper of class ``RobustScaler``"""
 
 
 class PolynomialFeaturesDF(
-    PolynomialTransformerWrapperDF[PolynomialFeatures],
-    native=PolynomialFeatures,
+    PolynomialTransformerWrapperDF[PolynomialFeatures], native=PolynomialFeatures
 ):
     """Stub for DF wrapper of class ``PolynomialFeatures``"""
 
 
-class NormalizerDF(
-    ColumnPreservingTransformerWrapperDF[Normalizer],
-    native=Normalizer,
-):
+class NormalizerDF(ColumnPreservingTransformerWrapperDF[Normalizer], native=Normalizer):
     """Stub for DF wrapper of class ``Normalizer``"""
 
 
-class BinarizerDF(
-    ColumnPreservingTransformerWrapperDF[Binarizer],
-    native=Binarizer,
-):
+class BinarizerDF(ColumnPreservingTransformerWrapperDF[Binarizer], native=Binarizer):
     """Stub for DF wrapper of class ``Binarizer``"""
 
 
 class KernelCentererDF(
-    ColumnPreservingTransformerWrapperDF[KernelCenterer],
-    native=KernelCenterer,
+    ColumnPreservingTransformerWrapperDF[KernelCenterer], native=KernelCenterer
 ):
     """Stub for DF wrapper of class ``KernelCenterer``"""
 
@@ -333,8 +352,7 @@ class QuantileTransformerDF(
 
 
 class PowerTransformerDF(
-    ColumnPreservingTransformerWrapperDF[PowerTransformer],
-    native=PowerTransformer,
+    ColumnPreservingTransformerWrapperDF[PowerTransformer], native=PowerTransformer
 ):
     """Stub for DF wrapper of class ``PowerTransformer``"""
 
@@ -347,15 +365,13 @@ class FunctionTransformerDF(
 
 
 class LabelEncoderDF(
-    ColumnPreservingTransformerWrapperDF[LabelEncoder],
-    native=LabelEncoder,
+    ColumnPreservingTransformerWrapperDF[LabelEncoder], native=LabelEncoder
 ):
     """Stub for DF wrapper of class ``LabelEncoder``"""
 
 
 class LabelBinarizerDF(
-    ColumnPreservingTransformerWrapperDF[LabelBinarizer],
-    native=LabelBinarizer,
+    ColumnPreservingTransformerWrapperDF[LabelBinarizer], native=LabelBinarizer
 ):
     """Stub for DF wrapper of class ``LabelBinarizer``"""
 
@@ -367,24 +383,17 @@ class MultiLabelBinarizerDF(
     """Stub for DF wrapper of class ``MultiLabelBinarizer``"""
 
 
-class OneHotEncoderDF(
-    OneHotEncoderWrapperDF,
-    native=OneHotEncoder,
-):
+class OneHotEncoderDF(OneHotEncoderWrapperDF, native=OneHotEncoder):
     """Stub for DF wrapper of class ``OneHotEncoder``"""
 
 
 class OrdinalEncoderDF(
-    ColumnPreservingTransformerWrapperDF[OrdinalEncoder],
-    native=OrdinalEncoder,
+    ColumnPreservingTransformerWrapperDF[OrdinalEncoder], native=OrdinalEncoder
 ):
     """Stub for DF wrapper of class ``OrdinalEncoder``"""
 
 
-class KBinsDiscretizerDF(
-    KBinsDiscretizerWrapperDF,
-    native=KBinsDiscretizer,
-):
+class KBinsDiscretizerDF(KBinsDiscretizerWrapperDF, native=KBinsDiscretizer):
     """Stub for DF wrapper of class ``KBinsDiscretizer``"""
 
 
@@ -395,8 +404,7 @@ class KBinsDiscretizerDF(
 
 
 class BernoulliRBMDF(
-    ComponentsDimensionalityReductionWrapperDF[BernoulliRBM],
-    native=BernoulliRBM,
+    ComponentsDimensionalityReductionWrapperDF[BernoulliRBM], native=BernoulliRBM
 ):
     """Stub for DF wrapper of class ``BernoulliRBM``"""
 
@@ -409,16 +417,12 @@ class DictionaryLearningDF(
 
 
 class FactorAnalysisDF(
-    ComponentsDimensionalityReductionWrapperDF[FactorAnalysis],
-    native=FactorAnalysis,
+    ComponentsDimensionalityReductionWrapperDF[FactorAnalysis], native=FactorAnalysis
 ):
     """Stub for DF wrapper of class ``FactorAnalysis``"""
 
 
-class FastICADF(
-    ComponentsDimensionalityReductionWrapperDF[FastICA],
-    native=FastICA,
-):
+class FastICADF(ComponentsDimensionalityReductionWrapperDF[FastICA], native=FastICA):
     """Stub for DF wrapper of class ``FastICA``"""
 
 
@@ -430,8 +434,7 @@ class GaussianRandomProjectionDF(
 
 
 class IncrementalPCADF(
-    ComponentsDimensionalityReductionWrapperDF[IncrementalPCA],
-    native=IncrementalPCA,
+    ComponentsDimensionalityReductionWrapperDF[IncrementalPCA], native=IncrementalPCA
 ):
     """Stub for DF wrapper of class ``IncrementalPCA``"""
 
@@ -461,20 +464,18 @@ class NMFDF(ComponentsDimensionalityReductionWrapperDF[NMF], native=NMF):
     """Stub for DF wrapper of class ``NMF``"""
 
 
-class PCADF(NComponentsDimensionalityReductionWrapperDF[PCA], native=PCA):
+class PCADF(PCAWrapperDF, native=PCA):
     """Stub for DF wrapper of class ``PCA``"""
 
 
 class SparseCoderDF(
-    ComponentsDimensionalityReductionWrapperDF[SparseCoder],
-    native=SparseCoder,
+    ComponentsDimensionalityReductionWrapperDF[SparseCoder], native=SparseCoder
 ):
     """Stub for DF wrapper of class ``SparseCoder``"""
 
 
 class SparsePCADF(
-    ComponentsDimensionalityReductionWrapperDF[SparsePCA],
-    native=SparsePCA,
+    ComponentsDimensionalityReductionWrapperDF[SparsePCA], native=SparsePCA
 ):
     """Stub for DF wrapper of class ``SparsePCA``"""
 
@@ -487,8 +488,7 @@ class SparseRandomProjectionDF(
 
 
 class TruncatedSVDDF(
-    ComponentsDimensionalityReductionWrapperDF[TruncatedSVD],
-    native=TruncatedSVD,
+    ComponentsDimensionalityReductionWrapperDF[TruncatedSVD], native=TruncatedSVD
 ):
     """Stub for DF wrapper of class ``TruncatedSVD``"""
 
@@ -500,8 +500,7 @@ class TruncatedSVDDF(
 
 
 class KernelPCADF(
-    NComponentsDimensionalityReductionWrapperDF[KernelPCA],
-    native=KernelPCA,
+    NComponentsDimensionalityReductionWrapperDF[KernelPCA], native=KernelPCA
 ):
     """Stub for DF wrapper of class ``KernelPCA``"""
 
@@ -514,15 +513,13 @@ class LocallyLinearEmbeddingDF(
 
 
 class NystroemDF(
-    NComponentsDimensionalityReductionWrapperDF[Nystroem],
-    native=Nystroem,
+    NComponentsDimensionalityReductionWrapperDF[Nystroem], native=Nystroem
 ):
     """Stub for DF wrapper of class ``Nystroem``"""
 
 
 class RBFSamplerDF(
-    NComponentsDimensionalityReductionWrapperDF[RBFSampler],
-    native=RBFSampler,
+    NComponentsDimensionalityReductionWrapperDF[RBFSampler], native=RBFSampler
 ):
     """Stub for DF wrapper of class ``RBFSampler``"""
 
@@ -542,8 +539,7 @@ class SkewedChi2SamplerDF(
 
 
 class VarianceThresholdDF(
-    FeatureSelectionWrapperDF[VarianceThreshold],
-    native=VarianceThreshold,
+    FeatureSelectionWrapperDF[VarianceThreshold], native=VarianceThreshold
 ):
     """Stub for DF wrapper of class ``VarianceThreshold``"""
 
@@ -557,23 +553,18 @@ class RFECVDF(FeatureSelectionWrapperDF[RFECV], native=RFECV):
 
 
 class SelectFromModelDF(
-    FeatureSelectionWrapperDF[SelectFromModel],
-    native=SelectFromModel,
+    FeatureSelectionWrapperDF[SelectFromModel], native=SelectFromModel
 ):
     """Stub for DF wrapper of class ``SelectFromModel``"""
 
 
 class SelectPercentileDF(
-    FeatureSelectionWrapperDF[SelectPercentile],
-    native=SelectPercentile,
+    FeatureSelectionWrapperDF[SelectPercentile], native=SelectPercentile
 ):
     """Stub for DF wrapper of class ``SelectPercentile``"""
 
 
-class SelectKBestDF(
-    FeatureSelectionWrapperDF[SelectKBest],
-    native=SelectKBest,
-):
+class SelectKBestDF(FeatureSelectionWrapperDF[SelectKBest], native=SelectKBest):
     """Stub for DF wrapper of class ``SelectKBest``"""
 
 
@@ -590,8 +581,7 @@ class SelectFweDF(FeatureSelectionWrapperDF[SelectFwe], native=SelectFwe):
 
 
 class GenericUnivariateSelectDF(
-    FeatureSelectionWrapperDF[GenericUnivariateSelect],
-    native=GenericUnivariateSelect,
+    FeatureSelectionWrapperDF[GenericUnivariateSelect], native=GenericUnivariateSelect
 ):
     """Stub for DF wrapper of class ``GenericUnivariateSelect``"""
 
@@ -613,7 +603,7 @@ __estimators = {
     if sym.endswith("DF")
     and sym not in __imported_estimators
     and not sym.startswith("_")
-}
+} | {CountVectorizerTx.__name__, TfidfVectorizerTx.__name__}
 
 if __estimators != set(__all__):
     raise RuntimeError(
