@@ -1,76 +1,72 @@
 """
 Core implementation of :mod:`sklearndf.transformation.wrapper`
 """
+
 import itertools
 import logging
 from abc import ABCMeta, abstractmethod
-from typing import (
-    Any,
-    Callable,
-    Generic,
-    Iterable,
-    List,
-    Optional,
-    Sequence,
-    TypeVar,
-    Union,
-    cast,
-)
+from collections.abc import Iterable, Sequence
+from typing import Any, Generic, Optional, TypeVar, Union, cast
 
 import numpy as np
 import numpy.typing as npt
 import pandas as pd
+from scipy import sparse
 from sklearn.base import TransformerMixin
 from sklearn.compose import ColumnTransformer
+from sklearn.decomposition import PCA
 from sklearn.impute import MissingIndicator, SimpleImputer
 from sklearn.kernel_approximation import AdditiveChi2Sampler
 from sklearn.manifold import Isomap
-from sklearn.preprocessing import KBinsDiscretizer, OneHotEncoder
+from sklearn.preprocessing import FunctionTransformer, KBinsDiscretizer, OneHotEncoder
 
 from pytools.api import AllTracker
 
-from ... import TransformerDF, __sklearn_1_0__, __sklearn_version__
+from ... import TransformerDF
+from ..._util import hstack_frames, is_sparse_frame, sparse_frame_density
 from ...wrapper import TransformerWrapperDF
 
 log = logging.getLogger(__name__)
 
 __all__ = [
-    "BaseDimensionalityReductionWrapperDF",
+    "AdditiveChi2SamplerWrapperDF",
+    "BaseMultiOutputWrapperDF",
     "BaseMultipleInputsPerOutputTransformerWrapperDF",
     "ColumnPreservingTransformerWrapperDF",
     "ColumnSubsetTransformerWrapperDF",
+    "ColumnTransformerSparseFrames",
+    "ColumnTransformerWrapperDF",
     "ComponentsDimensionalityReductionWrapperDF",
+    "EmbeddingWrapperDF",
     "FeatureSelectionWrapperDF",
+    "ImputerWrapperDF",
+    "IsomapWrapperDF",
+    "KBinsDiscretizerWrapperDF",
+    "MissingIndicatorWrapperDF",
     "NComponentsDimensionalityReductionWrapperDF",
     "NumpyTransformerWrapperDF",
-    "ColumnTransformerWrapperDF",
-    "IsomapWrapperDF",
-    "ImputerWrapperDF",
-    "MissingIndicatorWrapperDF",
-    "AdditiveChi2SamplerWrapperDF",
-    "KBinsDiscretizerWrapperDF",
-    "PolynomialTransformerWrapperDF",
     "OneHotEncoderWrapperDF",
+    "PCAWrapperDF",
+    "PolynomialTransformerWrapperDF",
+    "SingleColumnTransformerWrapperDF",
+    "VectorizerWrapperDF",
 ]
 
 
 #
-# type variables
+# Type variables
 #
 
-T_Transformer = TypeVar("T_Transformer", bound=TransformerMixin)
-
-# T_Imputer is needed because scikit-learn's _BaseImputer only exists from v0.22
-# onwards.
-# Once we drop support for sklearn 0.21, _BaseImputer can be used instead.
-# The following TypeVar helps to annotate availability of "add_indicator" and
-# "missing_values" attributes on an imputer instance for ImputerWrapperDF below.
 
 # noinspection PyProtectedMember
 from sklearn.impute._iterative import IterativeImputer
 
+# Once we drop support for sklearn 0.21, T_Imputer can be bound to _BaseImputer
 T_Imputer = TypeVar("T_Imputer", SimpleImputer, IterativeImputer)
 T_Polynomial = TypeVar("T_Polynomial", bound=TransformerMixin)
+T_Target = TypeVar("T_Target", bound=Union[pd.Series, pd.DataFrame, None])
+T_Transformer = TypeVar("T_Transformer", bound=TransformerMixin)
+T_Vectorizer = TypeVar("T_Vectorizer", bound=TransformerMixin)
 
 #
 # Ensure all symbols introduced below are included in __all__
@@ -99,13 +95,68 @@ class NumpyTransformerWrapperDF(
     def _adjust_X_type_for_delegate(
         self,
         X: pd.DataFrame,
-    ) -> npt.NDArray[Any]:
-        return cast(npt.NDArray[Any], X.values)
+    ) -> Union[npt.NDArray[Any], sparse.csr_matrix]:
+        return self._frame_or_series_to_array(X)
 
     def _adjust_y_type_for_delegate(
         self, y: Union[pd.Series, pd.DataFrame, None]
-    ) -> Optional[npt.NDArray[Any]]:
-        return None if y is None else cast(npt.NDArray[Any], y.values)
+    ) -> Union[npt.NDArray[Any], pd.arrays.SparseArray, sparse.csr_matrix, None]:
+        return None if y is None else self._frame_or_series_to_array(y)
+
+    def _frame_or_series_to_array(
+        self, df: Union[pd.Series, pd.DataFrame]
+    ) -> Union[npt.NDArray[Any], pd.arrays.SparseArray, sparse.csr_matrix]:
+        if df.ndim == 1:
+            return df.values
+
+        sparse_threshold = self._get_sparse_threshold()
+
+        if sparse_threshold > 0.0 and sparse_frame_density(df) < sparse_threshold:
+            return sparse.hstack(
+                [
+                    (
+                        sr.to_frame().sparse.to_coo()
+                        if isinstance(sr.dtype, pd.SparseDtype)
+                        else sr.values.reshape(-1, 1)
+                    )
+                    for _, sr in df.items()
+                ]
+            ).tocsr()
+
+        else:
+            return df.values
+
+    def _get_sparse_threshold(self) -> float:
+        # return the density below which _frame_or_series_to_array will
+        # convert series and data frames to sparse matrices instead of
+        # dense arrays
+        return 0.3
+
+
+class SingleColumnTransformerWrapperDF(
+    TransformerWrapperDF[T_Transformer], Generic[T_Transformer], metaclass=ABCMeta
+):
+    """
+    Abstract base class of DF wrappers for transformers that only accept single-column
+    inputs.
+    """
+
+    # noinspection PyPep8Naming
+    def _validate_parameter_types(
+        self,
+        X: Union[pd.Series, pd.DataFrame],
+        y: T_Target,
+        *,
+        expected_columns: pd.Index = None,
+    ) -> tuple[pd.DataFrame, T_Target]:
+        X, y = super()._validate_parameter_types(
+            X, y, expected_columns=expected_columns
+        )
+        if X.shape[1] != 1:
+            raise ValueError(
+                f"arg X expected to have exactly 1 column but has {X.shape[1]} columns"
+            )
+        return X, y
 
 
 class ColumnSubsetTransformerWrapperDF(
@@ -166,7 +217,7 @@ class BaseMultipleInputsPerOutputTransformerWrapperDF(
         )
 
 
-class BaseDimensionalityReductionWrapperDF(
+class BaseMultiOutputWrapperDF(
     BaseMultipleInputsPerOutputTransformerWrapperDF[T_Transformer],
     Generic[T_Transformer],
     metaclass=ABCMeta,
@@ -187,7 +238,7 @@ class BaseDimensionalityReductionWrapperDF(
 
 
 class NComponentsDimensionalityReductionWrapperDF(
-    BaseDimensionalityReductionWrapperDF[T_Transformer],
+    BaseMultiOutputWrapperDF[T_Transformer],
     Generic[T_Transformer],
     metaclass=ABCMeta,
 ):
@@ -201,15 +252,38 @@ class NComponentsDimensionalityReductionWrapperDF(
     _ATTR_N_COMPONENTS = "n_components"
 
     def _validate_delegate_estimator(self) -> None:
-        self._validate_delegate_attribute(attribute_name=self._ATTR_N_COMPONENTS)
+        self._validate_delegate_attribute(
+            attribute_name=(
+                NComponentsDimensionalityReductionWrapperDF._ATTR_N_COMPONENTS
+            )
+        )
 
     @property
     def _n_components_(self) -> int:
-        return cast(int, getattr(self.native_estimator, self._ATTR_N_COMPONENTS))
+        return cast(
+            int,
+            getattr(
+                self.native_estimator,
+                NComponentsDimensionalityReductionWrapperDF._ATTR_N_COMPONENTS,
+            ),
+        )
+
+
+class PCAWrapperDF(NComponentsDimensionalityReductionWrapperDF[PCA]):
+    """
+    DF wrapper for :class:`sklearn.decomposition.PCA`.
+
+    Uses attribute :attr:`~sklearn.decomposition.PCA.n_components_` to get the number
+    of components, which may be determined by the estimator during fitting.
+    """
+
+    @property
+    def _n_components_(self) -> int:
+        return cast(int, self._native_estimator.n_components_)
 
 
 class ComponentsDimensionalityReductionWrapperDF(
-    BaseDimensionalityReductionWrapperDF[T_Transformer],
+    BaseMultiOutputWrapperDF[T_Transformer],
     Generic[T_Transformer],
     metaclass=ABCMeta,
 ):
@@ -257,6 +331,52 @@ class FeatureSelectionWrapperDF(
         get_support = getattr(self.native_estimator, self._ATTR_GET_SUPPORT)
         return self.feature_names_in_[get_support()]
 
+    # noinspection PyPep8Naming
+
+
+class ColumnTransformerSparseFrames(
+    ColumnTransformer,  # type:ignore
+):
+    """
+    ColumnTransformer that returns sparse data frames instead of arrays if one or more
+    of its transformers return a sparse data frame.
+    """
+
+    # noinspection PyPep8Naming
+    def _hstack(
+        self,
+        Xs: list[Union[npt.NDArray[Any], sparse.spmatrix, pd.DataFrame]],
+        *,
+        n_samples: Optional[int] = None,
+    ) -> Union[npt.NDArray[Any], sparse.spmatrix, pd.DataFrame]:
+        if all(isinstance(X, pd.DataFrame) for X in Xs):
+            stacked: pd.DataFrame
+
+            if self.verbose_feature_names_out:
+                # Process all transformers in the ColumnTransformer
+                prefixes = [
+                    name
+                    for name, _, _ in self.transformers
+                    if _slice_not_empty(self.output_indices_[name])
+                ]
+                if self._remainder[2] and self.remainder != "drop":
+                    # Remainder columns exist and are not being dropped
+                    prefixes.append("remainder")
+                stacked = hstack_frames(Xs, prefixes=prefixes)
+            else:
+                stacked = hstack_frames(Xs)
+            self.sparse_output_ = is_sparse_frame(stacked)
+            return stacked
+        else:
+            if n_samples is None:
+                # Do not pass n_samples if we did not receive it, for backwards
+                # compatibility
+                # noinspection PyArgumentList
+                return super()._hstack(Xs)
+            else:
+                # Only pass n_samples if we receive it
+                return super()._hstack(Xs, n_samples=n_samples)
+
 
 class ColumnTransformerWrapperDF(
     TransformerWrapperDF[ColumnTransformer], metaclass=ABCMeta
@@ -286,10 +406,10 @@ class ColumnTransformerWrapperDF(
             not in ColumnTransformerWrapperDF.__SPECIAL_TRANSFORMERS
         ):
             raise ValueError(
-                f"unsupported value for arg remainder: ({column_transformer.remainder})"
+                f"unsupported value for arg remainder: {column_transformer.remainder!r}"
             )
 
-        non_compliant_transformers: List[str] = [
+        non_compliant_transformers: list[str] = [
             type(transformer).__name__
             for _, transformer, _ in column_transformer.transformers
             if not (
@@ -305,7 +425,7 @@ class ColumnTransformerWrapperDF(
                 f"{TransformerDF.__name__} or special values "
                 f'"{" and ".join(ColumnTransformerWrapperDF.__SPECIAL_TRANSFORMERS)}" '
                 "as valid transformers, but "
-                f'also got: {", ".join(non_compliant_transformers)}'
+                f"also got: {', '.join(non_compliant_transformers)}"
             )
 
     def _get_features_original(self) -> pd.Series:
@@ -331,9 +451,15 @@ class ColumnTransformerWrapperDF(
             input_column_names: npt.NDArray[Any]
             output_column_names: npt.NDArray[Any]
 
-            if df_transformer == ColumnTransformerWrapperDF.PASSTHROUGH:
+            if df_transformer == ColumnTransformerWrapperDF.PASSTHROUGH or (
+                # scikit-learn 1.4+ replaces 'passthrough' with a FunctionTransformer
+                # using the identity function (represented by `None`), so we need to
+                # check for that.
+                isinstance(df_transformer, FunctionTransformer)
+                and df_transformer.func is None
+            ):
                 # we may get positional indices for columns selected by the
-                # 'passthrough' transformer, and in that case so need to look up the
+                # 'passthrough' transformer, and in that case need to look up the
                 # associated column names
                 if all(isinstance(column, int) for column in columns):
                     output_column_names = self._get_features_in().to_numpy()[columns]
@@ -347,7 +473,7 @@ class ColumnTransformerWrapperDF(
                     f"{type(df_transformer).__name__}: {df_transformer!r}"
                 )
                 feature_names_original_: pd.Series = (
-                    df_transformer.feature_names_original_
+                    df_transformer._get_features_original()
                 )
                 if verbose_feature_names_out:
                     input_column_names = feature_names_original_.to_numpy()
@@ -379,7 +505,9 @@ class ColumnTransformerWrapperDF(
         )
 
 
-class ImputerWrapperDF(TransformerWrapperDF[T_Imputer], metaclass=ABCMeta):
+class ImputerWrapperDF(
+    TransformerWrapperDF[T_Imputer], Generic[T_Imputer], metaclass=ABCMeta
+):
     """
     DF wrapper for imputation transformers, e.g., :class:`sklearn.impute.SimpleImputer`.
     """
@@ -388,11 +516,11 @@ class ImputerWrapperDF(TransformerWrapperDF[T_Imputer], metaclass=ABCMeta):
         # get the columns that were dropped during imputation
         delegate_estimator = self.native_estimator
 
-        nan_mask: Union[List[bool], npt.NDArray[Any]] = []
+        nan_mask: Union[list[bool], npt.NDArray[Any]] = []
 
         def _nan_mask_from_statistics(
             stats: npt.NDArray[Any],
-        ) -> Union[List[bool], npt.NDArray[np.bool_]]:
+        ) -> Union[list[bool], npt.NDArray[np.bool_]]:
             if issubclass(stats.dtype.type, float):
                 return cast(npt.NDArray[np.bool_], np.isnan(stats))
             else:
@@ -447,11 +575,14 @@ class MissingIndicatorWrapperDF(
         features_original: npt.NDArray[Any] = self.feature_names_in_[
             self.native_estimator.features_
         ].values
-        features_out = pd.Index([f"{name}__missing" for name in features_original])
+        # noinspection SpellCheckingInspection
+        features_out = pd.Index(
+            [f"missingindicator_{name}" for name in features_original]
+        )
         return pd.Series(index=features_out, data=features_original)
 
 
-class IsomapWrapperDF(BaseDimensionalityReductionWrapperDF[Isomap], metaclass=ABCMeta):
+class IsomapWrapperDF(BaseMultiOutputWrapperDF[Isomap], metaclass=ABCMeta):
     """
     DF wrapper for :class:`sklearn.manifold.Isomap`.
     """
@@ -462,7 +593,7 @@ class IsomapWrapperDF(BaseDimensionalityReductionWrapperDF[Isomap], metaclass=AB
 
 
 class AdditiveChi2SamplerWrapperDF(
-    BaseDimensionalityReductionWrapperDF[AdditiveChi2Sampler], metaclass=ABCMeta
+    BaseMultiOutputWrapperDF[AdditiveChi2Sampler], metaclass=ABCMeta
 ):
     """
     DF wrapper for :class:`sklearn.kernel_approximation.AdditiveChi2Sampler`.
@@ -487,9 +618,10 @@ class PolynomialTransformerWrapperDF(
     """
 
     def _get_features_out(self) -> pd.Index:
-        return _get_native_feature_names_out(
-            feature_names_in_=self.feature_names_in_,
-            native_estimator=self.native_estimator,
+        return pd.Index(
+            self.native_estimator.get_feature_names_out(
+                self.feature_names_in_.to_numpy().astype(str)
+            )
         )
 
 
@@ -498,17 +630,15 @@ class OneHotEncoderWrapperDF(TransformerWrapperDF[OneHotEncoder], metaclass=ABCM
     DF wrapper for :class:`sklearn.preprocessing.OneHotEncoder`.
     """
 
-    def _validate_delegate_estimator(self) -> None:
-        if self.native_estimator.sparse:
-            raise NotImplementedError("sparse matrices not supported; use sparse=False")
-
     def _get_features_original(self) -> pd.Series:
         # Return the series mapping output column names to original column names.
 
         native_estimator: OneHotEncoder = self.native_estimator
         feature_names_in: pd.Index = self.feature_names_in_
-        feature_names_out: pd.Index = _get_native_feature_names_out(
-            feature_names_in_=feature_names_in, native_estimator=native_estimator
+        feature_names_out: pd.Index = pd.Index(
+            native_estimator.get_feature_names_out(
+                feature_names_in.to_numpy().astype(str)
+            )
         )
 
         def _adjust_n_features_in(n: npt.NDArray[np.int_]) -> None:
@@ -535,6 +665,26 @@ class OneHotEncoderWrapperDF(TransformerWrapperDF[OneHotEncoder], metaclass=ABCM
             [len(categories) for categories in native_estimator.categories_],
             dtype=np.int_,
         )
+
+        if self.max_categories is not None or self.min_frequency is not None:
+            # count number of infrequent categories per column
+            n_infrequent = np.array(
+                [
+                    (
+                        1
+                        if infrequent_categories_for_column is None
+                        else len(infrequent_categories_for_column)
+                    )
+                    for infrequent_categories_for_column in (
+                        self.native_estimator.infrequent_categories_
+                    )
+                ],
+                dtype=np.int_,
+            )
+            # all infrequent categories will be aggregated, so deduct number of
+            # infrequent categories, except one for the aggregated category
+            n_features_in -= n_infrequent - 1
+
         _adjust_n_features_in(n_features_in)
 
         feature_names_in_mapped = itertools.chain(
@@ -590,26 +740,64 @@ class KBinsDiscretizerWrapperDF(
             )
 
 
+class EmbeddingWrapperDF(
+    BaseMultiOutputWrapperDF[T_Transformer],
+    Generic[T_Transformer],
+    metaclass=ABCMeta,
+):
+    """
+    Base class of DF wrappers for dimensionality-reducing transformers.
+
+    The native transformer is considered to map all input columns to each output column.
+    """
+
+    def _get_n_outputs(self) -> int:
+        return cast(int, self.native_estimator.n_outputs_)
+
+    @property
+    def _n_components_(self) -> int:
+        return self.n_outputs_
+
+
 #
 # Helper functions
 #
 
 
-def _get_native_feature_names_out(
-    feature_names_in_: pd.Index, native_estimator: TransformerMixin
-) -> pd.Index:
-    # get the output feature names from a native transformer implementing
-    # method get_feature_names() (sklearn 0.x) or get_feature_names_out() (sklearn 1.x)
+class VectorizerWrapperDF(
+    SingleColumnTransformerWrapperDF[T_Transformer], Generic[T_Transformer]
+):
+    """
+    DF wrapper for vectorizers, specifically
+    :class:`~sklearn.feature_extraction.text.CountVectorizer` and
+    :class:`~sklearn.feature_extraction.text.TfidfVectorizer`.
+    """
 
-    get_feature_names_out_fn: Callable[[npt.NDArray[Any]], npt.NDArray[Any]]
-    if __sklearn_version__ >= __sklearn_1_0__:
-        # noinspection PyUnresolvedReferences
-        get_feature_names_out_fn = native_estimator.get_feature_names_out
-    else:
-        # noinspection PyUnresolvedReferences
-        get_feature_names_out_fn = native_estimator.get_feature_names
+    def _get_features_original(self) -> pd.Series:
+        return pd.Series(
+            index=self._get_features_out(), data=self._get_features_in()[0]
+        )
 
-    return pd.Index(get_feature_names_out_fn(feature_names_in_.to_numpy().astype(str)))
+    def _get_features_out(self) -> pd.Index:
+        try:
+            feature_names = self.native_estimator.get_feature_names_out()
+        except AttributeError:
+            try:
+                n_features = self.native_estimator.n_features
+            except AttributeError:
+                raise TypeError(
+                    f"native vectorizer {type(self.native_estimator).__name__} "
+                    "has no method get_feature_names_out() or attribute n_features"
+                )
+            else:
+                return pd.RangeIndex(n_features)
+        else:
+            return pd.Index(feature_names)
+
+    # noinspection PyPep8Naming
+    def _adjust_X_type_for_delegate(self, X: pd.DataFrame) -> npt.NDArray[Any]:
+        assert len(X.columns) == 1
+        return cast(npt.NDArray[Any], X.iloc[:, 0].values)
 
 
 #
@@ -617,3 +805,16 @@ def _get_native_feature_names_out(
 #
 
 __tracker.validate()
+
+
+#
+# Auxiliary functions
+#
+
+
+def _slice_not_empty(s: slice) -> bool:
+    if s.start is None:
+        return s.stop is None or s.stop > 0
+    if s.stop is None:
+        return True
+    return s.stop > s.start  # type: ignore[no-any-return]
