@@ -17,29 +17,15 @@ import inspect
 import logging
 import warnings
 from abc import ABCMeta
+from collections.abc import Iterable, Mapping, Sequence
 from functools import update_wrapper
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Generic,
-    Iterable,
-    List,
-    Mapping,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-)
+from typing import Any, Callable, Generic, Optional, TypeVar, Union, cast
 
 import numpy as np
 import numpy.typing as npt
 import pandas as pd
 import sklearn.utils.metaestimators as sklearn_meta
+from scipy import sparse
 from sklearn.base import (
     BaseEstimator,
     ClassifierMixin,
@@ -59,10 +45,13 @@ from sklearndf import (
     RegressorDF,
     SupervisedLearnerDF,
     TransformerDF,
-    __sklearn_1_0__,
+    __sklearn_1_6__,
     __sklearn_version__,
 )
 
+if __sklearn_version__ >= __sklearn_1_6__:
+    from sklearn.utils import Tags
+
 log = logging.getLogger(__name__)
 
 __all__ = [
@@ -84,6 +73,7 @@ __all__ = [
 
 T = TypeVar("T")
 T_Callable = TypeVar("T_Callable", bound=Callable[..., Any])
+T_Target = TypeVar("T_Target", bound=Optional[Union[pd.Series, pd.DataFrame]])
 
 T_NativeEstimator = TypeVar("T_NativeEstimator", bound=BaseEstimator)
 T_NativeTransformer = TypeVar("T_NativeTransformer", bound=TransformerMixin)
@@ -121,13 +111,13 @@ class EstimatorWrapperDFMeta(ABCMeta, Generic[T_NativeEstimator]):
     """
 
     #: the native class wrapped by the DF wrapper class
-    __wrapped__: Type[T_NativeEstimator]
+    __wrapped__: type[T_NativeEstimator]
 
     def __new__(
-        mcs: Type[EstimatorWrapperDFMeta[T_NativeEstimator]],
+        mcs: type[EstimatorWrapperDFMeta[T_NativeEstimator]],
         name: str,
-        bases: Tuple[type, ...],
-        namespace: Dict[str, Any],
+        bases: tuple[type, ...],
+        namespace: dict[str, Any],
         native: Optional[T_NativeEstimator] = None,
         **kwargs: Any,
     ) -> EstimatorWrapperDFMeta[T_NativeEstimator]:
@@ -145,7 +135,7 @@ class EstimatorWrapperDFMeta(ABCMeta, Generic[T_NativeEstimator]):
         if native is None:
             return cls
 
-        wrapper_cls = cast(Type[EstimatorWrapperDF[T_NativeEstimator]], cls)
+        wrapper_cls = cast(type[EstimatorWrapperDF[T_NativeEstimator]], cls)
 
         if not issubclass(native, wrapper_cls.__native_base_class__):
             raise TypeError(
@@ -183,7 +173,7 @@ class EstimatorWrapperDFMeta(ABCMeta, Generic[T_NativeEstimator]):
         return wrapper_cls
 
     @property
-    def native_estimator_type(cls) -> Type[T_NativeEstimator]:
+    def native_estimator_type(cls) -> type[T_NativeEstimator]:
         """
         The type of native estimator that instances of this wrapper class delegate to.
         """
@@ -212,22 +202,28 @@ class EstimatorWrapperDF(
     __native_base_class__ = BaseEstimator
     __ARG_FITTED_DELEGATE_CONTEXT = "__EstimatorWrapperDF_fitted"
 
+    #: The native estimator that this wrapper delegates to.
+    _native_estimator: T_NativeEstimator
+
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         """
-        :param args: positional arguments to use when initializing a new new delegate
+        :param args: positional arguments to use when initializing a new delegate
             estimator
-        :param kwargs: keyword arguments to use when initializing a new new delegate
+        :param kwargs: keyword arguments to use when initializing a new delegate
             estimator
         """
         super().__init__()
         self._features_in: Optional[pd.Index] = None
-        self._n_outputs: Optional[int] = None
+        self._outputs: Optional[list[str]] = None
 
         # check if a fitted estimator was passed by class method is_fitted
-        fitted_delegate_context: Tuple[T_NativeEstimator, pd.Index, int] = kwargs.get(
-            EstimatorWrapperDF.__ARG_FITTED_DELEGATE_CONTEXT, None
+        fitted_delegate_context = cast(
+            Optional[tuple[T_NativeEstimator, pd.Index, int]],
+            kwargs.get(EstimatorWrapperDF.__ARG_FITTED_DELEGATE_CONTEXT, None),
         )
 
+        _native_estimator: T_NativeEstimator
+
         if fitted_delegate_context is None:
             # create a new delegate estimator with the given parameters
             # noinspection PyProtectedMember
@@ -245,7 +241,7 @@ class EstimatorWrapperDF(
         self._validate_delegate_estimator()
 
     def __new__(
-        cls: Type[T_EstimatorWrapperDF], *args: Any, **kwargs: Any
+        cls: type[T_EstimatorWrapperDF], *args: Any, **kwargs: Any
     ) -> T_EstimatorWrapperDF:
         try:
             cls.__wrapped__
@@ -255,7 +251,7 @@ class EstimatorWrapperDF(
                 "need to specify class argument 'native' in class definition"
             )
         else:
-            return cast(Type[EstimatorDF], super()).__new__(cls)
+            return cast(type[EstimatorDF], super()).__new__(cls)
 
     @property
     def is_fitted(self) -> bool:
@@ -276,6 +272,11 @@ class EstimatorWrapperDF(
             super().feature_names_in_, warning_stacklevel=2
         )
 
+    @property
+    def n_features_in_(self) -> int:
+        """[see superclass]"""
+        return self._check_n_features_in(super().n_features_in_, warning_stacklevel=2)
+
     def _check_feature_names_in(
         self, wrapper_feature_names_in: pd.Index, *, warning_stacklevel: int
     ) -> pd.Index:
@@ -300,6 +301,30 @@ class EstimatorWrapperDF(
             )
         return wrapper_feature_names_in
 
+    def _check_n_features_in(
+        self, wrapper_n_features: int, *, warning_stacklevel: int
+    ) -> int:
+        # Check that the given number of features is the same as the number of features
+        # recorded by the native estimator, if present. Issue a warning if the number of
+        # features differ.
+        # Return the same number of features that were passed to this method.
+
+        # noinspection PyBroadException
+        try:
+            n_features_native = self.native_estimator.n_features_in_
+        except Exception:
+            return wrapper_n_features
+
+        if wrapper_n_features != n_features_native:
+            warnings.warn(
+                "conflicting number of features: "
+                "the number of features recorded by this estimator is "
+                f"{wrapper_n_features}, but the number of features recorded by "
+                f"the wrapped native estimator is {n_features_native}",
+                stacklevel=warning_stacklevel + 1,
+            )
+        return wrapper_n_features
+
     @property
     def _estimator_type(self) -> Optional[str]:
         try:
@@ -308,9 +333,14 @@ class EstimatorWrapperDF(
         except AttributeError:
             return None
 
+    if __sklearn_version__ >= __sklearn_1_6__:
+
+        def __sklearn_tags__(self) -> Tags:
+            return self.native_estimator.__sklearn_tags__()
+
     @classmethod
     def from_fitted(
-        cls: Type[T_EstimatorWrapperDF],
+        cls: type[T_EstimatorWrapperDF],
         estimator: T_NativeEstimator,
         features_in: pd.Index,
         n_outputs: int,
@@ -347,7 +377,7 @@ class EstimatorWrapperDF(
     # noinspection PyPep8Naming
     def fit(
         self: T_EstimatorWrapperDF,
-        X: pd.DataFrame,
+        X: Union[pd.DataFrame, pd.Series],
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         **fit_params: Any,
     ) -> T_EstimatorWrapperDF:
@@ -356,7 +386,7 @@ class EstimatorWrapperDF(
         self._reset_fit()
 
         try:
-            self._check_parameter_types(X, y)
+            X, y = self._validate_parameter_types(X, y)
             self._fit(X, y, **fit_params)
             self._post_fit(X, y, **fit_params)
 
@@ -367,20 +397,20 @@ class EstimatorWrapperDF(
         return self
 
     def _validate_delegate_estimator(self) -> None:
-        # no validation required by default; to be overloaded as needed
+        # Called as the last step of the estimator wrapper's constructor.
+        # No validation required by default; to be overloaded as needed.
         pass
 
     def _get_features_in(self) -> pd.Index:
         assert self._features_in is not None, "estimator is fitted"
         return self._features_in
 
-    def _get_n_outputs(self) -> int:
-        assert self._n_outputs is not None, "estimator is fitted"
-        return self._n_outputs
+    def _get_outputs(self) -> Optional[list[str]]:
+        return self._outputs
 
     def _reset_fit(self) -> None:
         self._features_in = None
-        self._n_outputs = None
+        self._outputs = None
 
     # noinspection PyPep8Naming
     def _fit(
@@ -406,24 +436,38 @@ class EstimatorWrapperDF(
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         **fit_params: Any,
     ) -> None:
-        self._features_in = X.columns.rename(self.COL_FEATURE_IN)
+        self._features_in = X.columns.rename(self.COL_FEATURE)
         if y is None:
-            self._n_outputs = 0
+            self._outputs = None
         elif isinstance(y, pd.Series):
-            self._n_outputs = 1
+            self._outputs = [y.name]
         else:
-            self._n_outputs = y.shape[1]
+            self._outputs = y.columns.tolist()
 
     # noinspection PyPep8Naming
-    def _check_parameter_types(
+    def _validate_parameter_types(
         self,
-        X: pd.DataFrame,
-        y: Optional[Union[pd.Series, pd.DataFrame]],
+        X: Union[pd.Series, pd.DataFrame],
+        y: T_Target,
         *,
-        expected_columns: pd.Index = None,
-    ) -> None:
-        if not isinstance(X, pd.DataFrame):
-            raise TypeError("arg X must be a DataFrame")
+        expected_columns: Optional[pd.Index] = None,
+    ) -> tuple[pd.DataFrame, T_Target]:
+        # Check that the X and y parameters are valid data frames and series,
+        # and return X as a data frame and y as a series or data frame.
+        #
+        # If X is a series, convert it to a data frame with a single column.
+        #
+        # If expected_columns is not None, check that the columns of X match
+        # the expected columns.
+
+        if isinstance(X, pd.Series):
+            if X.name is None:
+                raise ValueError(
+                    "the name of the series passed as arg X must not be None"
+                )
+            X = X.to_frame()
+        elif not isinstance(X, pd.DataFrame):
+            raise TypeError("arg X must be a DataFrame or a Series")
 
         if self.is_fitted:
             EstimatorWrapperDF._verify_df(
@@ -439,37 +483,50 @@ class EstimatorWrapperDF(
         if y is not None and not isinstance(y, (pd.Series, pd.DataFrame)):
             raise TypeError("arg y must be None, or a pandas series or data frame")
 
+        return X, y
+
     @staticmethod
     def _verify_df(
         df_name: str,
         df: pd.DataFrame,
         expected_columns: pd.Index,
-        expected_index: pd.Index = None,
+        expected_index: Optional[pd.Index] = None,
     ) -> None:
         def _verify_labels(axis: str, actual: pd.Index, expected: pd.Index) -> None:
-            error_message = f"{df_name} data frame does not have expected {axis}"
             missing_columns = expected.difference(actual)
             extra_columns = actual.difference(expected)
-            error_detail = []
+            error_detail: list[str] = []
+
+            # check that we have the expected number of columns
             if len(actual) != len(expected):
                 error_detail.append(
-                    f"expected {len(expected)} columns but got {len(actual)}"
+                    f"expected {len(expected)} elements but got {len(actual)}"
                 )
-                if len(missing_columns) > 0:
-                    error_detail.append(
-                        f"missing columns: "
-                        f"{', '.join(str(item) for item in missing_columns)}"
-                    )
-                if len(extra_columns) > 0:
-                    error_detail.append(
-                        f"extra columns: "
-                        f"{', '.join(str(item) for item in extra_columns)}"
-                    )
-                raise ValueError(f"{error_message} ({'; '.join(error_detail)})")
 
-        _verify_labels(axis="columns", actual=df.columns, expected=expected_columns)
+            # check that all the expected columns are in place
+            if len(missing_columns) > 0:
+                error_detail.append(
+                    f"missing elements: "
+                    f"{', '.join(str(item) for item in missing_columns)}"
+                )
+
+            # check that there are no unexpected columns
+            if len(extra_columns) > 0:
+                error_detail.append(
+                    f"extra elements: "
+                    f"{', '.join(str(item) for item in extra_columns)}"
+                )
+
+            # raise an exception if we have encountered any errors
+            if error_detail:
+                raise ValueError(
+                    f"{df_name} data frame does not have expected {axis} index "
+                    f"({'; '.join(error_detail)})"
+                )
+
+        _verify_labels(axis="column", actual=df.columns, expected=expected_columns)
         if expected_index is not None:
-            _verify_labels(axis="index", actual=df.index, expected=expected_index)
+            _verify_labels(axis="row", actual=df.index, expected=expected_index)
 
     def _validate_delegate_attribute(self, attribute_name: str) -> None:
         if not hasattr(self.native_estimator, attribute_name):
@@ -481,13 +538,13 @@ class EstimatorWrapperDF(
     # noinspection PyPep8Naming
     def _prepare_X_for_delegate(
         self, X: pd.DataFrame
-    ) -> Union[pd.DataFrame, npt.NDArray[Any]]:
+    ) -> Union[pd.DataFrame, npt.NDArray[Any], sparse.csr_matrix]:
         # convert X before passing it to the delegate estimator
         return self._adjust_X_type_for_delegate(self._adjust_X_columns_for_delegate(X))
 
     def _prepare_y_for_delegate(
         self, y: Optional[Union[pd.Series, pd.DataFrame]]
-    ) -> Any:
+    ) -> Optional[Union[pd.Series, pd.DataFrame, npt.NDArray[Any], sparse.csr_matrix]]:
         return self._adjust_y_type_for_delegate(y)
 
     # noinspection PyPep8Naming
@@ -507,14 +564,14 @@ class EstimatorWrapperDF(
     # noinspection PyPep8Naming
     def _adjust_X_type_for_delegate(
         self, X: pd.DataFrame
-    ) -> Union[pd.DataFrame, npt.NDArray[Any]]:
+    ) -> Union[pd.DataFrame, npt.NDArray[Any], sparse.csr_matrix]:
         # Convert X before passing it to the delegate estimator.
         # By default, does nothing, but can be overridden.
         return X
 
     def _adjust_y_type_for_delegate(
-        self, y: Union[pd.Series, pd.DataFrame, None]
-    ) -> Union[pd.Series, pd.DataFrame, npt.NDArray[Any], None]:
+        self, y: Optional[Union[pd.Series, pd.DataFrame]]
+    ) -> Optional[Union[pd.Series, pd.DataFrame, npt.NDArray[Any], sparse.csr_matrix]]:
         # convert y before passing it to the delegate estimator
         return y
 
@@ -538,23 +595,42 @@ class EstimatorWrapperDF(
         }
 
     def __getattr__(self, name: str) -> Any:
-        # get a non-private attribute of the delegate estimator
+        # This method is only called if the attribute name is not found in the
+        # instance's dictionary, and __getattribute__() has raised an AttributeError.
+
+        # For private attributes, give up and raise attribute error.
         if name.startswith("_"):
-            # raise attribute error
+            # The following will raise an AttributeError
             self.__getattribute__(name)
         else:
+            # For public attributes, try to get the attribute from the delegate
+            # estimator. If the attribute is not found, raise an attribute error.
             try:
                 return getattr(self._native_estimator, name)
             except AttributeError:
-                # raise attribute error
+                # The following will raise an AttributeError
                 self.__getattribute__(name)
 
     def __setattr__(self, name: str, value: Any) -> None:
-        # set a public attribute of the delegate estimator
+        # This method is called whenever an attribute assignment is attempted.
+
+        # For private attributes, set the attribute in this wrapper object.
         if name.startswith("_"):
             super().__setattr__(name, value)
         else:
-            setattr(self._native_estimator, name, value)
+            # For public attributes, set the attribute in this wrapper object only
+            # if it is already defined. Otherwise, set the attribute in the delegate
+            # estimator.
+
+            try:
+                self.__getattribute__(name)
+            except AttributeError:
+                # The attribute is not defined in this wrapper object, so set it in
+                # the delegate estimator.
+                setattr(self._native_estimator, name, value)
+            else:
+                # The attribute is defined in this wrapper object, so set it here.
+                super().__setattr__(name, value)
 
 
 @inheritdoc(match="[see superclass]")
@@ -575,7 +651,7 @@ class TransformerWrapperDF(
     def feature_names_out_(self) -> pd.Index:
         """[see superclass]"""
         return self._check_feature_names_out(
-            self._get_features_in(), super().feature_names_out_, warning_stacklevel=2
+            super().feature_names_out_, warning_stacklevel=2
         )
 
     @property
@@ -583,16 +659,14 @@ class TransformerWrapperDF(
         """[see superclass]"""
         feature_names_original_ = super().feature_names_original_
         self._check_feature_names_out(
-            self._get_features_in().values,
-            feature_names_original_.index,
-            warning_stacklevel=2,
+            feature_names_original_.index, warning_stacklevel=2
         )
         return feature_names_original_
 
     # noinspection PyPep8Naming
-    def transform(self, X: pd.DataFrame) -> pd.DataFrame:
+    def transform(self, X: Union[pd.Series, pd.DataFrame]) -> pd.DataFrame:
         """[see superclass]"""
-        self._check_parameter_types(X, None)
+        X, _ = self._validate_parameter_types(X, None)
 
         transformed = self._transform(X)
 
@@ -602,13 +676,16 @@ class TransformerWrapperDF(
 
     # noinspection PyPep8Naming
     def fit_transform(
-        self, X: pd.DataFrame, y: Optional[pd.Series] = None, **fit_params: Any
+        self,
+        X: Union[pd.Series, pd.DataFrame],
+        y: Optional[pd.Series] = None,
+        **fit_params: Any,
     ) -> pd.DataFrame:
         """[see superclass]"""
         self._reset_fit()
 
         try:
-            self._check_parameter_types(X, y)
+            X, y = self._validate_parameter_types(X, y)
             transformed = self._fit_transform(X, y, **fit_params)
             self._post_fit(X, y, **fit_params)
 
@@ -623,9 +700,11 @@ class TransformerWrapperDF(
         )
 
     # noinspection PyPep8Naming
-    def inverse_transform(self, X: pd.DataFrame) -> pd.DataFrame:
+    def inverse_transform(self, X: Union[pd.Series, pd.DataFrame]) -> pd.DataFrame:
         """[see superclass]"""
-        self._check_parameter_types(X, None, expected_columns=self.feature_names_out_)
+        X, _ = self._validate_parameter_types(
+            X, None, expected_columns=self.feature_names_out_
+        )
 
         transformed = self._inverse_transform(X)
 
@@ -634,18 +713,12 @@ class TransformerWrapperDF(
         )
 
     def _check_feature_names_out(
-        self,
-        feature_names_in: npt.NDArray[Any],
-        wrapper_feature_names_out: pd.Index,
-        *,
-        warning_stacklevel: int,
+        self, wrapper_feature_names_out: pd.Index, *, warning_stacklevel: int
     ) -> pd.Index:
-        if __sklearn_version__ < __sklearn_1_0__:
-            return wrapper_feature_names_out
         # noinspection PyBroadException
         try:
             native_feature_names_out = self.native_estimator.get_feature_names_out(
-                feature_names_in
+                self._get_features_in().values
             )
         except Exception:
             return wrapper_feature_names_out
@@ -695,11 +768,13 @@ class TransformerWrapperDF(
 
     @staticmethod
     def _transformed_to_df(
-        transformed: Union[pd.DataFrame, npt.NDArray[Any]],
+        transformed: Union[pd.DataFrame, npt.NDArray[Any], sparse.spmatrix],
         index: pd.Index,
         columns: pd.Index,
     ) -> pd.DataFrame:
         if isinstance(transformed, pd.DataFrame):
+            if transformed.columns.name != EstimatorDF.COL_FEATURE:
+                transformed = transformed.rename_axis(columns=EstimatorDF.COL_FEATURE)
             # noinspection PyProtectedMember
             TransformerWrapperDF._verify_df(
                 df_name="transformed",
@@ -708,23 +783,28 @@ class TransformerWrapperDF(
                 expected_index=index,
             )
             return transformed
+        elif isinstance(transformed, sparse.spmatrix):
+            return pd.DataFrame.sparse.from_spmatrix(
+                data=transformed, index=index, columns=columns
+            )
         else:
             return pd.DataFrame(data=transformed, index=index, columns=columns)
 
     # noinspection PyPep8Naming
-    def _transform(self, X: pd.DataFrame) -> npt.NDArray[Any]:
-        # noinspection PyUnresolvedReferences
+    def _transform(
+        self, X: pd.DataFrame
+    ) -> Union[npt.NDArray[Any], sparse.csr_matrix, pd.DataFrame]:
         return cast(
-            npt.NDArray[Any],
+            Union[npt.NDArray[Any], sparse.csr_matrix, pd.DataFrame],
             self.native_estimator.transform(self._prepare_X_for_delegate(X)),
         )
 
     # noinspection PyPep8Naming
     def _fit_transform(
         self, X: pd.DataFrame, y: Optional[pd.Series], **fit_params: Any
-    ) -> npt.NDArray[Any]:
+    ) -> Union[npt.NDArray[Any], sparse.csr_matrix, pd.DataFrame]:
         return cast(
-            npt.NDArray[Any],
+            Union[npt.NDArray[Any], sparse.csr_matrix, pd.DataFrame],
             self.native_estimator.fit_transform(
                 self._prepare_X_for_delegate(X),
                 self._prepare_y_for_delegate(y),
@@ -768,10 +848,10 @@ class LearnerWrapperDF(
 
     # noinspection PyPep8Naming
     def predict(
-        self, X: pd.DataFrame, **predict_params: Any
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
-        self._check_parameter_types(X, None)
+        X, _ = self._validate_parameter_types(X, None)
 
         # noinspection PyUnresolvedReferences
         return self._prediction_to_series_or_frame(
@@ -785,20 +865,54 @@ class LearnerWrapperDF(
     def _prediction_to_series_or_frame(
         self, X: pd.DataFrame, y: Union[npt.NDArray[Any], pd.Series, pd.DataFrame]
     ) -> Union[pd.Series, pd.DataFrame]:
-        if isinstance(y, pd.Series) or isinstance(y, pd.DataFrame):
-            # if we already have a series or data frame, check it and return it
-            # unchanged
-            return y
+        if len(y) != len(X):
+            raise ValueError(
+                f"length of prediction ({len(y)}) does not match length of X ({len(X)})"
+            )
+
+        outputs: Optional[list[str]] = self._get_outputs()
+
+        if y.ndim == 1:
+            if outputs is None:
+                # in case predict() was called without first calling fit(),
+                # no outputs are known, so we use the default output name
+                outputs = [self.COL_PREDICTION]
+            elif len(outputs) != 1:
+                raise ValueError(
+                    f"expected {len(outputs)} predictions, but got 1 prediction"
+                )
+        elif y.ndim == 2:
+            if outputs is None:
+                # in case predict() was called without first calling fit(),
+                # no outputs are known, so we use default output names
+                outputs = [
+                    f"{self.COL_PREDICTION}_{output}" for output in range(y.shape[1])
+                ]
+            elif y.shape[1] != len(outputs):
+                raise ValueError(
+                    f"expected {len(outputs)} predictions, "
+                    f"but got {y.shape[1]} prediction{'' if y.shape[1] == 1 else 's'}"
+                )
+        else:
+            raise ValueError(
+                f"got {y.ndim}-dimensional prediction, "
+                f"but expected 1- or 2-dimensional prediction"
+            )
+
+        if isinstance(y, pd.Series):
+            return y.rename(outputs[0])
+        elif isinstance(y, pd.DataFrame):
+            return y.set_axis(outputs, axis=1)
         elif isinstance(y, np.ndarray):
             if len(y) == len(X):
                 # predictions are usually provided as a numpy array the same length as X
                 if y.ndim == 1:
                     # single-output predictions yield a numpy array of shape (n_samples)
-                    return pd.Series(data=y, name=self.COL_PREDICTION, index=X.index)
+                    return pd.Series(data=y, name=outputs[0], index=X.index)
                 if y.ndim == 2:
                     # multi-output predictions yield a numpy array of shape (n_samples,
                     # n_outputs)
-                    return pd.DataFrame(data=y, index=X.index)
+                    return pd.DataFrame(data=y, columns=outputs, index=X.index)
             raise TypeError(
                 f"unexpected shape of numpy array returned as prediction: {y.shape}"
             )
@@ -823,10 +937,13 @@ class SupervisedLearnerWrapperDF(
 
     # noinspection PyPep8Naming
     def score(
-        self, X: pd.DataFrame, y: pd.Series, sample_weight: Optional[pd.Series] = None
+        self,
+        X: Union[pd.Series, pd.DataFrame],
+        y: pd.Series,
+        sample_weight: Optional[pd.Series] = None,
     ) -> float:
         """[see superclass]"""
-        self._check_parameter_types(X, y)
+        X, y = self._validate_parameter_types(X, y)
         if y is None:
             raise ValueError("arg y must not be None")
         if sample_weight is not None and not isinstance(sample_weight, pd.Series):
@@ -858,7 +975,10 @@ class RegressorWrapperDF(
 
     # noinspection PyPep8Naming
     def score(
-        self, X: pd.DataFrame, y: pd.Series, sample_weight: Optional[pd.Series] = None
+        self,
+        X: Union[pd.Series, pd.DataFrame],
+        y: pd.Series,
+        sample_weight: Optional[pd.Series] = None,
     ) -> float:
         """[see superclass]"""
         return cast(
@@ -881,25 +1001,21 @@ class ClassifierWrapperDF(
 
     __native_base_class__ = ClassifierMixin
 
-    @property
-    def classes_(self) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
-        """[see superclass]"""
-        self.ensure_fitted()
-        # noinspection PyUnresolvedReferences
+    def _get_classes(self) -> Union[npt.NDArray[Any], list[npt.NDArray[Any]]]:
         return cast(
-            Union[npt.NDArray[Any], List[npt.NDArray[Any]]],
+            Union[npt.NDArray[Any], list[npt.NDArray[Any]]],
             self._native_estimator.classes_,
         )
 
     # noinspection PyPep8Naming
     def predict_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
+    ) -> Union[pd.DataFrame, list[pd.DataFrame]]:
         """[see superclass]"""
 
         self._ensure_delegate_method("predict_proba")
 
-        self._check_parameter_types(X, None)
+        X, _ = self._validate_parameter_types(X, None)
 
         # noinspection PyUnresolvedReferences
         return self._prediction_with_class_labels(
@@ -911,13 +1027,13 @@ class ClassifierWrapperDF(
 
     # noinspection PyPep8Naming
     def predict_log_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
+    ) -> Union[pd.DataFrame, list[pd.DataFrame]]:
         """[see superclass]"""
 
         self._ensure_delegate_method("predict_log_proba")
 
-        self._check_parameter_types(X, None)
+        X, _ = self._validate_parameter_types(X, None)
 
         # noinspection PyUnresolvedReferences
         return self._prediction_with_class_labels(
@@ -929,13 +1045,13 @@ class ClassifierWrapperDF(
 
     # noinspection PyPep8Naming
     def decision_function(
-        self, X: pd.DataFrame, **predict_params: Any
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
 
         self._ensure_delegate_method("decision_function")
 
-        self._check_parameter_types(X, None)
+        X, _ = self._validate_parameter_types(X, None)
 
         # noinspection PyUnresolvedReferences
         return self._prediction_with_class_labels(
@@ -957,18 +1073,17 @@ class ClassifierWrapperDF(
         self,
         X: pd.DataFrame,
         prediction: Union[
-            pd.Series, pd.DataFrame, List[npt.NDArray[Any]], npt.NDArray[Any]
+            pd.Series, pd.DataFrame, list[npt.NDArray[Any]], npt.NDArray[Any]
         ],
         classes: Optional[Sequence[Any]] = None,
-    ) -> Union[pd.Series, pd.DataFrame, List[pd.DataFrame]]:
-
+    ) -> Union[pd.Series, pd.DataFrame, list[pd.DataFrame]]:
         if classes is None:
             classes = getattr(self.native_estimator, "classes_", None)
             if classes is None:
                 classes = pd.RangeIndex(self._get_n_outputs())
 
         if isinstance(prediction, pd.DataFrame):
-            return prediction.set_axis(classes, axis=1, inplace=False)
+            return prediction.set_axis(classes, axis=1)
 
         elif isinstance(prediction, np.ndarray):
             if len(prediction) == len(X):
@@ -994,7 +1109,10 @@ class ClassifierWrapperDF(
 
     # noinspection PyPep8Naming
     def score(
-        self, X: pd.DataFrame, y: pd.Series, sample_weight: Optional[pd.Series] = None
+        self,
+        X: Union[pd.Series, pd.DataFrame],
+        y: pd.Series,
+        sample_weight: Optional[pd.Series] = None,
     ) -> float:
         """[see superclass]"""
         return cast(
@@ -1025,17 +1143,16 @@ class ClusterWrapperDF(
         super().__init__(*args, **kwargs)
         self._x_index: Optional[pd.Index] = None
 
-    @property
-    def labels_(self) -> pd.Series:
-        """[see superclass]"""
-        self.ensure_fitted()
-        raw_labels = self._native_estimator.labels_
-
-        return pd.Series(data=raw_labels, name=self.COL_LABELS, index=self._x_index)
+    def _get_labels(self) -> pd.Series:
+        return pd.Series(
+            data=self._native_estimator.labels_,
+            name=self.COL_LABELS,
+            index=self._x_index,
+        )
 
     def fit_predict(
         self,
-        X: pd.DataFrame,
+        X: Union[pd.Series, pd.DataFrame],
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         **fit_predict_params: Any,
     ) -> Union[pd.Series, pd.DataFrame]:
@@ -1044,7 +1161,10 @@ class ClusterWrapperDF(
         self._reset_fit()
 
         try:
-            self._check_parameter_types(X, y)
+            X, y = self._validate_parameter_types(X, y)
+
+            # fitting a clusterer produces a single output column for labels
+            self._outputs = [ClusterWrapperDF.COL_LABELS]
 
             # Ignore a PyCharm warning that is caused by scikit-learn incorrectly
             # omitting optional arguments from the abstract method declaration
@@ -1102,7 +1222,14 @@ class MetaEstimatorWrapperDF(
     copy of the `X` and `y` parameters, so we need to make sure that these are converted
     back to data frames.
 
-    This class covers three variants used in sklearn:
+    We also ensure that the embedded estimator is a native sci-kit learn estimator.
+    If it is a DF estimator, we replace it with its wrapped native estimator and
+    re-assign it to the attribute `estimator` or `base_estimator` (depending on the
+    meta-estimator).
+    In that case, we issue a warning that the wrapped estimator is being used instead
+    of the DF version.
+
+    This class covers three variants used in scikit-learn:
 
     - one delegate estimator in attribute `estimator`
     - one delegate estimator in attribute `base_estimator`
@@ -1110,37 +1237,54 @@ class MetaEstimatorWrapperDF(
     """
 
     def _validate_delegate_estimator(self) -> None:
-        def _unwrap_estimator(estimator: BaseEstimator) -> BaseEstimator:
-            native_estimator = (
-                estimator.native_estimator
-                if isinstance(estimator, EstimatorWrapperDF)
-                else estimator
-            )
-            # noinspection PyProtectedMember
-            if isinstance(
-                native_estimator, (EstimatorDF, sklearn_meta._BaseComposition)
-            ) or not isinstance(native_estimator, (RegressorMixin, ClassifierMixin)):
-                raise TypeError(
-                    "sklearndf meta-estimators only accept simple regressors and "
-                    f"classifiers, but got: {type(estimator).__name__}"
-                )
-            return cast(BaseEstimator, native_estimator)
+        substituted: list[str] = []
 
-        delegate_estimator = self.native_estimator
+        estimator = getattr(self, "estimator", None)
+        if estimator is not None:
+            self.estimator = self._native_learner(estimator)
+            substituted.append("estimator")
 
-        if hasattr(delegate_estimator, "estimator"):
-            delegate_estimator.estimator = _unwrap_estimator(
-                delegate_estimator.estimator
-            )
-        elif hasattr(delegate_estimator, "base_estimator"):
-            delegate_estimator.base_estimator = _unwrap_estimator(
-                delegate_estimator.base_estimator
-            )
-        elif hasattr(delegate_estimator, "estimators"):
-            delegate_estimator.estimators = [
-                (name, _unwrap_estimator(estimator))
-                for name, estimator in delegate_estimator.estimators
+        base_estimator = getattr(self, "base_estimator", None)
+        # attribute base_estimator is deprecated as of scikit-learn 1.2, with the
+        # default value of "deprecated"
+        if base_estimator is not None and base_estimator != "deprecated":
+            self.base_estimator = self._native_learner(base_estimator)
+            substituted.append("base_estimator")
+
+        estimators = getattr(self, "estimators", None)
+        if estimators is not None:
+            self.estimators = [
+                (name, self._native_learner(estimator))
+                for name, estimator in estimators
             ]
+            substituted.append("estimators")
+
+        if substituted:
+            warnings.warn(
+                f"the following attributes of {type(self).__name__} "
+                f"have been replaced with their native scikit-learn counterparts: "
+                f"{', '.join(substituted)}",
+                stacklevel=-2,
+            )
+
+    @staticmethod
+    def _native_learner(
+        estimator_wrapper: BaseEstimator,
+    ) -> Union[RegressorMixin, ClassifierMixin]:
+        native_estimator: BaseEstimator = (
+            estimator_wrapper.native_estimator
+            if isinstance(estimator_wrapper, EstimatorWrapperDF)
+            else estimator_wrapper
+        )
+        # noinspection PyProtectedMember
+        if isinstance(
+            native_estimator, (EstimatorDF, sklearn_meta._BaseComposition)
+        ) or not isinstance(native_estimator, (RegressorMixin, ClassifierMixin)):
+            raise TypeError(
+                "sklearndf meta-estimators only accept simple regressors and "
+                f"classifiers, but got: {type(estimator_wrapper).__name__}"
+            )
+        return native_estimator
 
 
 #
@@ -1149,17 +1293,15 @@ class MetaEstimatorWrapperDF(
 
 
 def _mirror_attributes(
-    wrapper_class: Type[EstimatorWrapperDF[T_NativeEstimator]],
-    native_estimator: Type[T_NativeEstimator],
+    wrapper_class: type[EstimatorWrapperDF[T_NativeEstimator]],
+    native_estimator: type[T_NativeEstimator],
     wrapper_module: str,
 ) -> None:
-
     wrapper_name = wrapper_class.__name__
-    wrapper_attributes: Set[str] = set(dir(wrapper_class))
+    wrapper_attributes: set[str] = set(dir(wrapper_class))
 
     for name, member in vars(native_estimator).items():
-
-        if member is None or name.startswith("_") or name in wrapper_attributes:
+        if member is None or name in wrapper_attributes:
             continue
 
         alias = _make_alias(
@@ -1176,7 +1318,8 @@ def _mirror_attributes(
 
 def _make_alias(
     wrapper_module: str, wrapper_name: str, name: str, delegate_cls: type, delegate: Any
-) -> Union[Callable[..., Any], property, None]:
+) -> Optional[Union[Callable[..., Any], property]]:
+
     if inspect.isfunction(delegate):
         return _make_method_alias(
             wrapper_module=wrapper_module,
@@ -1255,8 +1398,8 @@ def _update_wrapper(
 
 
 def _update_class_docstring(
-    df_estimator_type: Type[EstimatorWrapperDF[T_NativeEstimator]],
-    sklearn_native_estimator_type: Type[T_NativeEstimator],
+    df_estimator_type: type[EstimatorWrapperDF[T_NativeEstimator]],
+    sklearn_native_estimator_type: type[T_NativeEstimator],
 ) -> None:
     base_doc = sklearn_native_estimator_type.__doc__
 
@@ -1266,7 +1409,7 @@ def _update_class_docstring(
     base_doc_lines = base_doc.split("\n")
 
     # use the first paragraph as the tag line
-    tag_lines: List[str] = []
+    tag_lines: list[str] = []
     for line in base_doc_lines:
         # end of paragraph reached?
         stripped = line.strip()
