@@ -1,22 +1,13 @@
 """
 DF wrapper classes for stacking estimators.
 """
+
 from __future__ import annotations
 
 import logging
 from abc import ABCMeta, abstractmethod
-from typing import (
-    Any,
-    Callable,
-    Generic,
-    List,
-    Optional,
-    Sequence,
-    Tuple,
-    TypeVar,
-    Union,
-    cast,
-)
+from collections.abc import Sequence
+from typing import Any, Callable, Generic, Optional, TypeVar, Union, cast
 
 import numpy as np
 import numpy.typing as npt
@@ -77,6 +68,7 @@ __tracker = AllTracker(globals())
 # Stacking Estimator wrappers
 #
 
+
 # noinspection PyPep8Naming
 @inheritdoc(match="""[see superclass]""")
 class StackingEstimatorWrapperDF(
@@ -98,7 +90,7 @@ class StackingEstimatorWrapperDF(
 
     def fit(
         self: T_StackingEstimatorWrapperDF,
-        X: pd.DataFrame,
+        X: Union[pd.DataFrame, pd.Series],
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         **fit_params: Any,
     ) -> T_StackingEstimatorWrapperDF:
@@ -115,16 +107,18 @@ class StackingEstimatorWrapperDF(
                 return self
 
         native: T_NativeSupervisedLearner = self.native_estimator
-        estimators: Sequence[Tuple[str, BaseEstimator]] = native.estimators
+        estimators: Sequence[tuple[str, BaseEstimator]] = native.estimators
         final_estimator: BaseEstimator = native.final_estimator
 
         try:
             native.estimators = [
                 (
                     name,
-                    self._make_stackable_learner_df(estimator)
-                    if isinstance(estimator, SupervisedLearnerDF)
-                    else estimator,
+                    (
+                        self._make_stackable_learner_df(estimator)
+                        if isinstance(estimator, SupervisedLearnerDF)
+                        else estimator
+                    ),
                 )
                 for name, estimator in native.estimators
             ]
@@ -153,10 +147,10 @@ class StackingEstimatorWrapperDF(
     ) -> SupervisedLearnerNPDF[T_SupervisedLearnerDF]:
         pass
 
-    def _get_estimators_features_out(self) -> List[str]:
+    def _get_estimators_features_out(self) -> list[str]:
         return [name for name, estimator in self.estimators if estimator != "drop"]
 
-    def _get_final_estimator_features_in(self) -> List[str]:
+    def _get_final_estimator_features_in(self) -> list[str]:
         names = self._get_estimators_features_out()
         if self.passthrough:
             return [*names, *self.estimators_[0].feature_names_in_]
@@ -180,7 +174,7 @@ class StackingClassifierWrapperDF(
 
         return LogisticRegressionDF()
 
-    def _get_estimators_features_out(self) -> List[str]:
+    def _get_estimators_features_out(self) -> list[str]:
         classes = self.native_estimator.classes_
         names = super()._get_estimators_features_out()
         if len(classes) > 2:
@@ -256,7 +250,7 @@ class _StackableSupervisedLearnerDF(
     @subsdoc(pattern="", replacement="", using=SupervisedLearnerDF.fit)
     def fit(
         self: T_StackableSupervisedLearnerDF,
-        X: pd.DataFrame,
+        X: Union[pd.Series, pd.DataFrame],
         y: Optional[npt.NDArray[Any]] = None,
         **fit_params: Any,
     ) -> T_StackableSupervisedLearnerDF:
@@ -270,7 +264,9 @@ class _StackableSupervisedLearnerDF(
         replacement="predictions as a numpy array",
         using=SupervisedLearnerDF.predict,
     )
-    def predict(self, X: pd.DataFrame, **predict_params: Any) -> npt.NDArray[Any]:
+    def predict(
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
+    ) -> npt.NDArray[Any]:
         """[see SupervisedLearnerDF.predict]"""
         return cast(
             npt.NDArray[Any],
@@ -278,20 +274,29 @@ class _StackableSupervisedLearnerDF(
         )
 
     # noinspection PyPep8Naming
-    @subsdoc(pattern="", replacement="", using=SupervisedLearnerDF.score)
     def score(
         self,
-        X: pd.DataFrame,
-        y: npt.NDArray["np.floating[Any]"],
+        X: Union[pd.Series, pd.DataFrame],
+        y: npt.NDArray[np.floating[Any]],
         sample_weight: Optional[pd.Series] = None,
     ) -> float:
         """[see SupervisedLearnerDF.score]"""
         return self.delegate.score(X, self._convert_y_to_series(X, y), sample_weight)
 
+    score.__doc__ = SupervisedLearnerDF.score.__doc__
+
     def _get_features_in(self) -> pd.Index:
         # noinspection PyProtectedMember
         return self.delegate._get_features_in()
 
+    def _get_n_features_in(self) -> int:
+        # noinspection PyProtectedMember
+        return self.delegate._get_n_features_in()
+
+    def _get_outputs(self) -> Optional[list[str]]:
+        # noinspection PyProtectedMember
+        return self.delegate._get_outputs()
+
     def _get_n_outputs(self) -> int:
         # noinspection PyProtectedMember
         return self.delegate._get_n_outputs()
@@ -320,8 +325,8 @@ class _StackableSupervisedLearnerDF(
 
     @staticmethod
     def _convert_prediction_to_numpy(
-        prediction: Union[pd.DataFrame, List[pd.DataFrame]]
-    ) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
+        prediction: Union[pd.DataFrame, list[pd.DataFrame]],
+    ) -> Union[npt.NDArray[Any], list[npt.NDArray[Any]]]:
         if isinstance(prediction, list):
             return [proba.values for proba in prediction]
         else:
@@ -333,29 +338,27 @@ class _StackableSupervisedLearnerDF(
 class _StackableClassifierDF(_StackableSupervisedLearnerDF[ClassifierDF], ClassifierDF):
     """[see superclass]"""
 
-    @property
-    def classes_(self) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
-        """[see superclass]"""
-        return self.delegate.classes_
+    def _get_classes(self) -> Union[npt.NDArray[Any], list[npt.NDArray[Any]]]:
+        return self.delegate._get_classes()
 
     def predict_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
+    ) -> Union[npt.NDArray[Any], list[npt.NDArray[Any]]]:
         """[see superclass]"""
         return self._convert_prediction_to_numpy(
             self.delegate.predict_proba(X, **predict_params)
         )
 
     def predict_log_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
+    ) -> Union[npt.NDArray[Any], list[npt.NDArray[Any]]]:
         """[see superclass]"""
         return self._convert_prediction_to_numpy(
             self.delegate.predict_log_proba(X, **predict_params)
         )
 
     def decision_function(
-        self, X: pd.DataFrame, **predict_params: Any
+        self, X: Union[pd.Series, pd.DataFrame], **predict_params: Any
     ) -> npt.NDArray[np.floating[Any]]:
         """[see superclass]"""
         return cast(
