@@ -1,26 +1,36 @@
 import re
 import sys
-from distutils import version
-from typing import Any, Dict, Iterable, List, Optional, Set, Type, Union
+from collections.abc import Iterable
+from types import ModuleType
+from typing import Optional, Union
 
 import pandas as pd
 import sklearn
 from sklearn.base import BaseEstimator
+from sklearn.compose import ColumnTransformer
+from sklearn.pipeline import FeatureUnion
 
-from sklearndf import EstimatorDF, LearnerDF, TransformerDF
+from sklearndf import EstimatorDF, LearnerDF, TransformerDF, __sklearn_version__
+from sklearndf.pipeline.wrapper import FeatureUnionSparseFrames
+from sklearndf.transformation.wrapper import ColumnTransformerSparseFrames
 from sklearndf.wrapper import EstimatorWrapperDF
 
-Module: type = Any
+OVERRIDDEN_SKLEARN_CLASSES = {
+    ColumnTransformerSparseFrames: ColumnTransformer,
+    FeatureUnionSparseFrames: FeatureUnion,
+}
 
 
-def find_all_classes(*modules: Module) -> Set[Type[EstimatorWrapperDF]]:
-    """ Finds all Class members in given module/modules. """
-    types: Set[Type[EstimatorWrapperDF]] = set()
+def find_all_classes(
+    *modules: ModuleType,
+) -> set[type[EstimatorWrapperDF[BaseEstimator]]]:
+    """Finds all Class members in given module/modules."""
+    types: set[type[EstimatorWrapperDF[BaseEstimator]]] = set()
 
-    def _add_classes_from_module(_m: Module) -> None:
-        for member in vars(module).values():
+    def _add_classes_from_module(_m: ModuleType) -> None:
+        member: type[EstimatorWrapperDF[BaseEstimator]]
+        for member in vars(_m).values():
             if isinstance(member, type):
-                member: Type[EstimatorWrapperDF]
                 types.add(member)
 
     for module in modules:
@@ -29,8 +39,8 @@ def find_all_classes(*modules: Module) -> Set[Type[EstimatorWrapperDF]]:
     return types
 
 
-def find_all_submodules(parent_module: Module) -> Set[Module]:
-    """ Finds all submodules for a parent module. """
+def find_all_submodules(parent_module: ModuleType) -> set[ModuleType]:
+    """Finds all submodules for a parent module."""
     parent_name = f"{parent_module.__name__}."
     return {
         module
@@ -40,11 +50,16 @@ def find_all_submodules(parent_module: Module) -> Set[Module]:
 
 
 def sklearn_delegate_classes(
-    module: Module,
-) -> Dict[Type[BaseEstimator], Type[EstimatorWrapperDF]]:
-    """ Creates a dictionary mapping from sklearndf -> sklearn classes. """
+    module: ModuleType,
+) -> dict[type[BaseEstimator], type[EstimatorWrapperDF[BaseEstimator]]]:
+    """
+    Create a dictionary mapping sklearn classes to their corresponding sklearndf
+    classes.
+    """
     return {
-        df_class.__wrapped__: df_class
+        OVERRIDDEN_SKLEARN_CLASSES.get(
+            df_class.__wrapped__, df_class.__wrapped__
+        ): df_class
         for df_class in find_all_classes(module)
         # we only consider non-abstract wrapper classes wrapping a specific native class
         if issubclass(df_class, EstimatorWrapperDF) and hasattr(df_class, "__wrapped__")
@@ -52,30 +67,30 @@ def sklearn_delegate_classes(
 
 
 def iterate_classes(
-    from_modules: Union[Module, Iterable[Module]],
+    from_modules: Union[ModuleType, Iterable[ModuleType]],
     matching: str,
     excluding: Optional[Union[str, Iterable[str]]] = None,
-) -> Iterable[Type[EstimatorWrapperDF]]:
-    """ Helper to return all classes with matching name from Python module(s) """
+) -> list[type[EstimatorWrapperDF[BaseEstimator]]]:
+    """Helper to return all classes with matching name from Python module(s)"""
 
     if not isinstance(from_modules, Iterable):
         from_modules = (from_modules,)
 
-    if excluding and not isinstance(excluding, str):
+    if excluding is not None and not isinstance(excluding, str):
         excluding = "|".join(f"({exclude_pattern})" for exclude_pattern in excluding)
 
-    return (
+    return [
         m
         for m in find_all_classes(*from_modules)
         if re.match(matching, m.__name__)
-        and not (excluding and re.match(excluding, m.__name__))
-    )
+        and ((excluding is None) or not re.match(excluding, m.__name__))
+    ]
 
 
 def get_sklearndf_wrapper_class(
-    to_wrap: Type[BaseEstimator], from_module=None
-) -> Type[EstimatorWrapperDF]:
-    """ Helper to return the wrapped counterpart for a sklearn class """
+    to_wrap: type[BaseEstimator], from_module: ModuleType
+) -> type[EstimatorWrapperDF[BaseEstimator]]:
+    """Helper to return the wrapped counterpart for a sklearn class"""
     try:
         return sklearn_delegate_classes(from_module)[to_wrap]
 
@@ -85,10 +100,8 @@ def get_sklearndf_wrapper_class(
         ) from cause
 
 
-def check_expected_not_fitted_error(estimator: EstimatorDF):
-    """ Check if transformers & learners raise NotFittedError (since sklearn 0.22)"""
-    if version.LooseVersion(sklearn.__version__) <= "0.21":
-        return
+def check_expected_not_fitted_error(estimator: EstimatorDF) -> None:
+    """Check if transformers & learners raise NotFittedError"""
 
     test_x = pd.DataFrame(data=list(range(10)))
 
