@@ -2,12 +2,14 @@
 Test module for PipelineDF inspired by:
 https://github.com/scikit-learn/scikit-learn/blob/master/sklearn/tests/test_pipeline.py
 """
+
 from __future__ import annotations
 
 import shutil
 import time
+from collections.abc import Mapping
 from tempfile import mkdtemp
-from typing import Any, Dict, Mapping, cast
+from typing import Any, cast
 
 import joblib
 import numpy as np
@@ -15,14 +17,15 @@ import numpy.typing as npt
 import pandas as pd
 import pytest
 from numpy.testing import assert_array_equal, assert_no_warnings
+from pandas.testing import assert_frame_equal
 from sklearn import clone
 from sklearn.base import BaseEstimator, TransformerMixin, is_classifier, is_regressor
 from sklearn.feature_selection import f_classif
 
 from sklearndf.classification import SVCDF, LogisticRegressionDF
-from sklearndf.pipeline import PipelineDF
+from sklearndf.pipeline import FeatureUnionDF, PipelineDF
 from sklearndf.regression import DummyRegressorDF, LassoDF, LinearRegressionDF
-from sklearndf.transformation import SelectKBestDF, SimpleImputerDF
+from sklearndf.transformation import OneHotEncoderDF, SelectKBestDF, SimpleImputerDF
 from sklearndf.transformation.wrapper import ColumnPreservingTransformerWrapperDF
 
 
@@ -60,10 +63,10 @@ class NoTransformer(
     def fit(self, X: Any, y: Any = None, **fit_params: Any) -> NoTransformer:
         return self
 
-    def get_params(self, deep: bool = False) -> Dict[str, Any]:
+    def get_params(self, deep: bool = False) -> dict[str, Any]:
         return {"a": self.a, "b": self.b}
 
-    def set_params(self, a: Any = None, **params: Dict[str, Any]) -> NoTransformer:
+    def set_params(self, a: Any = None, **params: dict[str, Any]) -> NoTransformer:
         self.a = a
         return self
 
@@ -133,7 +136,7 @@ def test_pipeline_df_memory(
         pipe = PipelineDF([("tx", clone(tx)), ("svc", clf)])
         cached_pipe = PipelineDF([("tx", tx), ("svc", clf)], memory=memory)
 
-        # Memoize the transformer at the first fit
+        # Memorize the transformer at the first fit
         cached_pipe.fit(iris_features, iris_target_sr)
         pipe.fit(iris_features, iris_target_sr)
         # Get the time stamp of the transformer in the cached pipeline
@@ -216,14 +219,12 @@ def test_pipeline_df__init() -> None:
     with pytest.raises(TypeError):
         PipelineDF()
 
-    # Check that we can't instantiate pipelines with objects without fit
-    # method
+    # Check that we can't instantiate pipelines with objects without a `fit` method
     with pytest.raises(
-        TypeError,
+        ValueError,
         match=(
-            "Last step of Pipeline should implement fit "
-            "or be the string 'passthrough'"
-            ".*NoFit.*"
+            "expected final step 'clf' to be an EstimatorDF or passthrough, "
+            "but found an instance of NoFit"
         ),
     ):
         PipelineDF([("clf", NoFit())])
@@ -298,12 +299,97 @@ def test_pipeline_df_raise_set_params_error() -> None:
     pipe = PipelineDF([("cls", LinearRegressionDF())])
 
     with pytest.raises(
-        ValueError, match=r"Invalid parameter fake for estimator Pipeline"
+        ValueError,
+        match=r"Invalid parameter (fake|'fake') for estimator Pipeline",
     ):
         pipe.set_params(fake="nope")
 
     # nested model check
     with pytest.raises(
-        ValueError, match=r"Invalid parameter fake for estimator Pipeline"
+        ValueError,
+        match=r"Invalid parameter (fake|'fake') for estimator Pipeline",
     ):
         pipe.set_params(fake__estimator="nope")
+
+
+@pytest.mark.parametrize(  # type: ignore
+    argnames="sparse_output", argvalues=[True, False]
+)
+def test_feature_union(
+    test_data_categorical: pd.DataFrame, sparse_output: bool
+) -> None:
+    # the expected column dtype, depending on arg sparse
+    dtype_expected = (
+        pd.SparseDtype(np.float64, fill_value=0) if sparse_output else np.float64
+    )
+
+    # apply the test data to a simple feature union
+    feature_union = FeatureUnionDF(
+        [
+            ("one_hot", OneHotEncoderDF(drop="first", sparse_output=sparse_output)),
+        ]
+    )
+    transformed = feature_union.fit_transform(test_data_categorical)
+
+    # assert that all columns of the data frame are sparse
+    assert all(
+        col_dtype == dtype_expected for col_dtype in transformed.dtypes
+    ), f"all columns should be of type {dtype_expected}: {transformed.dtypes}"
+
+    # if the output is sparse, make it dense
+    if sparse_output:
+        transformed = transformed.sparse.to_dense()
+
+    # assert that the one-hot encoding is as expected
+    assert_frame_equal(
+        transformed,
+        pd.DataFrame(
+            dict(
+                one_hot__a_yes=[1.0, 1.0, 0.0],
+                one_hot__b_green=[0.0, 0.0, 1.0],
+                one_hot__b_red=[1.0, 0.0, 0.0],
+                one_hot__c_father=[0.0, 1.0, 0.0],
+                one_hot__c_mother=[0.0, 0.0, 1.0],
+            ),
+        ).rename_axis(columns="feature"),
+    )
+
+    # test the passthrough option introduced in sklearn 1.1
+
+    # apply the test data to a simple feature union
+    feature_union = FeatureUnionDF(
+        [
+            ("one_hot", OneHotEncoderDF(drop="first", sparse_output=sparse_output)),
+            ("pass", "passthrough"),
+            ("pass_again", "passthrough"),
+        ],
+    )
+    transformed = feature_union.fit_transform(test_data_categorical)
+
+    # assert that all one-hot-encoded columns of the data frame are sparse
+    assert all(
+        dtype == (dtype_expected if column.startswith("one_hot__") else np.object_)
+        for column, dtype in zip(transformed.columns, transformed.dtypes)
+    )
+
+    # assert that the one-hot encoding is as expected
+    assert_frame_equal(
+        transformed,
+        pd.concat(
+            [
+                pd.DataFrame(
+                    dict(
+                        one_hot__a_yes=[1.0, 1.0, 0.0],
+                        one_hot__b_green=[0.0, 0.0, 1.0],
+                        one_hot__b_red=[1.0, 0.0, 0.0],
+                        one_hot__c_father=[0.0, 1.0, 0.0],
+                        one_hot__c_mother=[0.0, 0.0, 1.0],
+                    ),
+                    dtype=dtype_expected,
+                ),
+                test_data_categorical.add_prefix("pass__"),
+                test_data_categorical.add_prefix("pass_again__"),
+            ],
+            axis=1,
+        ).rename_axis(columns="feature"),
+    )
