@@ -1,25 +1,30 @@
 # inspired by:
 # https://github.com/scikit-learn/scikit-learn/blob/master/sklearn/tests/test_base.py
+
+import re
 from typing import Any
 
 import numpy as np
 import pytest
 import scipy.sparse as sp
+import sklearn
 from numpy.testing import assert_array_equal
-from sklearn import clone
 from sklearn.base import BaseEstimator, is_classifier
 from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor
 from sklearn.model_selection import GridSearchCV
 from sklearn.pipeline import Pipeline
+from sklearn.utils import estimator_html_repr
 
 from pytools.expression import freeze, make_expression
 from pytools.expression.atomic import Id
 
+from sklearndf import __sklearn_1_6__, __sklearn_version__
 from sklearndf.classification import SVCDF, DecisionTreeClassifierDF
 from sklearndf.clustering.wrapper import KMeansBaseWrapperDF
 from sklearndf.pipeline import PipelineDF
-from sklearndf.pipeline.wrapper import FeatureUnionWrapperDF
-from sklearndf.transformation import OneHotEncoderDF
+from sklearndf.regression import RandomForestRegressorDF
+from sklearndf.transformation import OneHotEncoderDF, SimpleImputerDF
+from sklearndf.transformation.wrapper import ImputerWrapperDF
 from sklearndf.wrapper import (
     ClassifierWrapperDF,
     EstimatorWrapperDF,
@@ -69,13 +74,13 @@ def test_clone() -> None:
     # (which, in this case, is the current state of the estimator),
     # and check that the obtained copy is a correct deep copy.
 
-    encoder = OneHotEncoderDF(drop="first", sparse=False)
+    encoder = OneHotEncoderDF(drop="first", sparse_output=False)
     new_encoder = encoder.clone()
     assert encoder is not new_encoder
     assert encoder.get_params() == new_encoder.get_params()
 
-    encoder = OneHotEncoderDF(handle_unknown="ignore", sparse=False)
-    new_encoder = clone(encoder)
+    encoder = OneHotEncoderDF(handle_unknown="ignore", sparse_output=False)
+    new_encoder = sklearn.clone(encoder)
 
     assert encoder is not new_encoder
 
@@ -86,7 +91,7 @@ def test_clone_2() -> None:
     # make a copy of its original state. Then we check that the copy doesn't
     # have the specific attribute we manually added to the initial estimator.
 
-    encoder = OneHotEncoderDF(drop="first", sparse=False)
+    encoder = OneHotEncoderDF(drop="first", sparse_output=False)
 
     encoder.own_attribute = "test"
     new_encoder = encoder.clone()
@@ -115,7 +120,9 @@ def test_clone_nan() -> None:
 
 def test_clone_sparse_matrices() -> None:
     sparse_matrix_classes = [
-        getattr(sp, name) for name in dir(sp) if name.endswith("_matrix")
+        getattr(sp, name)
+        for name in dir(sp)
+        if name.endswith("_matrix") and name != "_matrix"
     ]
 
     for cls in sparse_matrix_classes:
@@ -161,6 +168,45 @@ def test_str() -> None:
     str(my_estimator)
 
 
+def test_html_repr() -> None:
+    # store the original display config
+    display_original = sklearn.get_config()["display"]
+
+    # set the display config to use diagrams
+    sklearn.set_config(display="diagram")
+
+    try:
+        pipeline_df = PipelineDF(
+            [
+                (
+                    "preprocess",
+                    PipelineDF(
+                        [
+                            ("impute", SimpleImputerDF()),
+                        ]
+                    ),
+                ),
+                ("rf", RandomForestRegressorDF(n_estimators=120)),
+            ]
+        )
+
+        def _replace_ids(_html: str) -> str:
+            # scikit-learn generates new ids on subsequent calls to estimator_html_repr,
+            # so we replace them with a placeholder
+            return re.sub(
+                r'(?<=id-)\d+|(?:(?<=sk-)|(?<=id=")|(?<=for="))\w+(?:-\w+)*', "#", _html
+            )
+
+        assert _replace_ids(pipeline_df._repr_html_()) == _replace_ids(
+            estimator_html_repr(pipeline_df)
+        )
+
+    finally:
+        # reset the display config to its original value
+        sklearn.set_config(display=display_original)
+        pass
+
+
 def test_get_params() -> None:
     test = DummyEstimator2DF(DummyEstimator3DF(), DummyEstimator3DF())
 
@@ -254,7 +300,7 @@ def test_native_class_validation() -> None:
     ):
 
         class MismatchedNativeClass4(
-            FeatureUnionWrapperDF, native=RandomForestRegressor
+            ImputerWrapperDF[Any], native=RandomForestRegressor
         ):
             pass
 
@@ -268,3 +314,25 @@ def test_native_class_validation() -> None:
 
         class MismatchedNativeClass5(PipelineDF, native=RandomForestRegressor):
             pass
+
+
+@pytest.mark.skipif(  # type: ignore[misc]
+    __sklearn_version__ < __sklearn_1_6__,
+    reason="This test requires scikit-learn 1.6 or later",
+)
+def test_sklearn_tags() -> None:
+    # Check that sklearn tags are correctly set for the DummyEstimatorDF
+    estimator = DummyEstimatorDF()
+    tags = estimator.__sklearn_tags__()
+
+    from sklearn.utils import InputTags
+
+    assert tags.estimator_type is None
+    assert isinstance(tags.input_tags, InputTags)
+    assert tags.transformer_tags is None
+    assert tags.classifier_tags is None
+    assert tags.regressor_tags is None
+    assert not tags.array_api_support
+    assert not tags.no_validation
+    assert not tags.non_deterministic
+    assert tags.requires_fit
