@@ -1,5 +1,5 @@
 from itertools import chain
-from typing import Any, Dict, Type
+from typing import Any
 
 import numpy as np
 import pandas as pd
@@ -10,10 +10,12 @@ from sklearn.multioutput import ClassifierChain, MultiOutputClassifier
 import sklearndf.classification as classification
 from sklearndf import (
     ClassifierDF,
-    __sklearn_0_22__,
-    __sklearn_1_0__,
+    __sklearn_1_5__,
+    __sklearn_1_6__,
+    __sklearn_1_8__,
     __sklearn_version__,
 )
+from sklearndf.classification.wrapper import ThresholdClassifierWrapperDF
 from test.sklearndf import check_expected_not_fitted_error, iterate_classes
 
 CLASSIFIERS_TO_TEST = iterate_classes(
@@ -27,17 +29,20 @@ def test_classifier_count() -> None:
     n = len(CLASSIFIERS_TO_TEST)
 
     print(f"Testing {n} classifiers.")
-    if __sklearn_version__ < __sklearn_0_22__:
-        assert n == 38
-    elif __sklearn_version__ < __sklearn_1_0__:
-        assert n == 40
-    else:
+
+    if __sklearn_version__ < __sklearn_1_5__:
         assert n == 41
+    elif __sklearn_version__ < __sklearn_1_6__:
+        assert n == 43
+    elif __sklearn_version__ < __sklearn_1_8__:
+        assert n == 44
+    else:
+        pytest.fail(f"Unexpected scikit-learn version: {__sklearn_version__}")
 
 
-CLASSIFIER_INIT_PARAMETERS: Dict[str, Dict[str, Any]] = {
+CLASSIFIER_INIT_PARAMETERS: dict[str, dict[str, Any]] = {
     "CalibratedClassifierCVDF": {
-        "base_estimator": classification.RandomForestClassifierDF()
+        "estimator": classification.RandomForestClassifierDF()
     },
     "ClassifierChainDF": {"base_estimator": classification.RandomForestClassifierDF()},
     "MultiOutputClassifierDF": {"estimator": classification.RandomForestClassifierDF()},
@@ -59,6 +64,16 @@ CLASSIFIER_INIT_PARAMETERS: Dict[str, Dict[str, Any]] = {
             ("AdaBoost", classification.AdaBoostClassifierDF()),
         )
     },
+    "TunedThresholdClassifierCVDF": {
+        "estimator": classification.RandomForestClassifierDF()
+    },
+    "FixedThresholdClassifierDF": {
+        "estimator": classification.RandomForestClassifierDF(),
+        "threshold": 0.5,
+    },
+    "SelfTrainingClassifierDF": {
+        "estimator": classification.RandomForestClassifierDF()
+    },
 }
 
 
@@ -71,16 +86,15 @@ CLASSIFIERS_PARTIAL_FIT = [
     classification.GaussianNBDF,
     classification.ComplementNBDF,
     classification.MultiOutputClassifierDF,
+    classification.CategoricalNBDF,
 ]
-if __sklearn_version__ >= __sklearn_0_22__:
-    CLASSIFIERS_PARTIAL_FIT.append(classification.CategoricalNBDF)
 
 
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearndf_cls", argvalues=CLASSIFIERS_TO_TEST
 )
 def test_wrapped_fit_predict(
-    sklearndf_cls: Type[ClassifierDF],
+    sklearndf_cls: type[ClassifierDF],
     iris_features: pd.DataFrame,
     iris_target_sr: pd.Series,
     iris_targets_df: pd.DataFrame,
@@ -88,7 +102,7 @@ def test_wrapped_fit_predict(
 ) -> None:
     """Test fit & predict & predict[_log]_proba of wrapped sklearn classifiers"""
     # noinspection PyArgumentList
-    parameters: Dict[str, Any] = CLASSIFIER_INIT_PARAMETERS.get(
+    parameters: dict[str, Any] = CLASSIFIER_INIT_PARAMETERS.get(
         sklearndf_cls.__name__, {}
     )
     # noinspection PyArgumentList
@@ -97,6 +111,7 @@ def test_wrapped_fit_predict(
     assert is_classifier(classifier)
 
     is_chain = isinstance(classifier.native_estimator, ClassifierChain)
+    is_threshold = isinstance(classifier, ThresholdClassifierWrapperDF)
 
     is_multi_output = isinstance(classifier.native_estimator, MultiOutputClassifier)
     check_expected_not_fitted_error(estimator=classifier)
@@ -106,6 +121,10 @@ def test_wrapped_fit_predict(
         # classification can act as input to the next classification
         classes = set(range(iris_targets_binary_df.shape[1]))
         classifier.fit(X=iris_features, y=iris_targets_binary_df)
+    elif is_threshold:
+        # for threshold classifiers, the classes are binary
+        classes = {0, 1}
+        classifier.fit(X=iris_features, y=iris_targets_binary_df.iloc[:, 0])
     elif is_multi_output:
         classes = set(
             chain(
@@ -124,7 +143,8 @@ def test_wrapped_fit_predict(
 
     # test predictions data-type, length and values
     assert isinstance(
-        predictions, pd.DataFrame if is_multi_output or is_chain else pd.Series
+        predictions,
+        pd.DataFrame if is_multi_output or is_chain else pd.Series,
     )
     assert len(predictions) == len(iris_target_sr)
     assert np.all(predictions.isin(classes))
@@ -138,9 +158,27 @@ def test_wrapped_fit_predict(
 
             if is_multi_output:
                 assert isinstance(predictions, list)
+                assert classifier.output_names_ == iris_targets_df.columns.tolist()
                 assert classifier.n_outputs_ == len(predictions)
             else:
-                assert classifier.n_outputs_ == predictions.shape[1] if is_chain else 1
+                if is_chain:
+                    # for classifier chains and threshold classifiers, we have a binary
+                    # output
+                    assert (
+                        classifier.output_names_
+                        == iris_targets_binary_df.columns.tolist()
+                    )
+                    assert classifier.n_outputs_ == predictions.shape[1]
+                elif is_threshold:
+                    # for threshold classifiers, we have a single binary output
+                    assert classifier.output_names_ == [
+                        iris_targets_binary_df.columns[0]
+                    ]
+                    assert classifier.n_outputs_ == 1
+                else:
+                    assert classifier.output_names_ == [iris_target_sr.name]
+                    assert classifier.n_outputs_ == 1
+
                 predictions = [predictions]
 
             for prediction in predictions:
@@ -159,7 +197,7 @@ def test_wrapped_fit_predict(
     argnames="sklearndf_cls", argvalues=CLASSIFIERS_PARTIAL_FIT
 )
 def test_wrapped_partial_fit(
-    sklearndf_cls: Type[ClassifierDF],
+    sklearndf_cls: type[ClassifierDF],
     iris_features: pd.DataFrame,
     iris_target_sr: pd.Series,
     iris_targets_df: pd.DataFrame,
