@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Type
+from typing import Any
 
 import pandas as pd
 import pytest
@@ -6,14 +6,7 @@ from sklearn.base import BaseEstimator, is_regressor
 from sklearn.multioutput import MultiOutputRegressor, RegressorChain
 
 import sklearndf.regression
-from sklearndf import (
-    RegressorDF,
-    TransformerDF,
-    __sklearn_0_22__,
-    __sklearn_0_23__,
-    __sklearn_1_0__,
-    __sklearn_version__,
-)
+from sklearndf import RegressorDF, TransformerDF, __sklearn_1_8__, __sklearn_version__
 from sklearndf.regression import (
     SVRDF,
     IsotonicRegressionDF,
@@ -29,7 +22,7 @@ from test.sklearndf import check_expected_not_fitted_error, iterate_classes
 
 # noinspection PyTypeChecker
 # ignore false alert about module type
-REGRESSORS_TO_TEST: List[Type[EstimatorWrapperDF[BaseEstimator]]] = iterate_classes(
+REGRESSORS_TO_TEST: list[type[EstimatorWrapperDF[BaseEstimator]]] = iterate_classes(
     from_modules=sklearndf.regression,
     matching=r".*DF",
     excluding=[RegressorDF.__name__, TransformerDF.__name__, r".*WrapperDF"],
@@ -40,17 +33,13 @@ def test_regressor_count() -> None:
     n = len(REGRESSORS_TO_TEST)
 
     print(f"Testing {n} regressors.")
-    if __sklearn_version__ < __sklearn_0_22__:
-        assert n == 49
-    elif __sklearn_version__ < __sklearn_0_23__:
-        assert n == 50
-    elif __sklearn_version__ < __sklearn_1_0__:
-        assert n == 54
+    if __sklearn_version__ < __sklearn_1_8__:
+        assert n == 55
     else:
-        assert n == 56
+        pytest.fail(f"Unexpected scikit-learn version: {__sklearn_version__}")
 
 
-DEFAULT_REGRESSOR_PARAMETERS: Dict[str, Dict[str, Any]] = {
+DEFAULT_REGRESSOR_PARAMETERS: dict[str, dict[str, Any]] = {
     "MultiOutputRegressorDF": dict(estimator=RandomForestRegressorDF()),
     "MultiOutputRegressorDF_partial_fit": dict(estimator=SGDRegressorDF()),
     "RegressorChainDF": dict(base_estimator=RandomForestRegressorDF()),
@@ -64,6 +53,12 @@ DEFAULT_REGRESSOR_PARAMETERS: Dict[str, Dict[str, Any]] = {
             ("Linear", LinearRegressionDF()),
         )
     ),
+    # the rank of Y is 1, so n_components needs to be 1
+    "CCADF": dict(n_components=1),
+    # the rank of Y is 1, so n_components needs to be 1
+    "PLSCanonicalDF": dict(n_components=1),
+    # use a solver that is still supported with scipy 1.11
+    "QuantileRegressorDF": dict(solver="highs"),
 }
 
 REGRESSORS_PARTIAL_FIT = [
@@ -78,13 +73,13 @@ REGRESSORS_PARTIAL_FIT = [
     argnames="sklearndf_cls", argvalues=REGRESSORS_TO_TEST
 )
 def test_wrapped_fit_predict(
-    sklearndf_cls: Type[RegressorDF],
-    boston_features: pd.DataFrame,
-    boston_target_sr: pd.Series,
-    boston_target_df: pd.DataFrame,
+    sklearndf_cls: type[RegressorDF],
+    diabetes_features: pd.DataFrame,
+    diabetes_target_sr: pd.Series,
+    diabetes_target_df: pd.DataFrame,
 ) -> None:
     """Test fit & predict of wrapped sklearn regressors"""
-    parameters: Dict[str, Any] = DEFAULT_REGRESSOR_PARAMETERS.get(
+    parameters: dict[str, Any] = DEFAULT_REGRESSOR_PARAMETERS.get(
         sklearndf_cls.__name__, {}
     )
 
@@ -100,41 +95,41 @@ def test_wrapped_fit_predict(
         or isinstance(regressor.native_estimator, MultiOutputRegressor)
         or isinstance(regressor.native_estimator, RegressorChain)
     ):
-        regressor.fit(X=boston_features, y=boston_target_df)
+        regressor.fit(X=diabetes_features, y=diabetes_target_df)
 
     else:
         if isinstance(regressor, IsotonicRegressionDF):
             # fit will fail when we have more than one feature
             with pytest.raises(ValueError):
-                regressor.fit(X=boston_features, y=boston_target_sr)
+                regressor.fit(X=diabetes_features, y=diabetes_target_sr)
             # eliminate all features except one then continue testing
-            boston_features = boston_features.loc[:, ["LSTAT"]]
+            diabetes_features = diabetes_features.loc[:, "bmi"]
 
-        regressor.fit(X=boston_features, y=boston_target_sr)
+        regressor.fit(X=diabetes_features, y=diabetes_target_sr)
 
-    predictions = regressor.predict(X=boston_features)
+    predictions = regressor.predict(X=diabetes_features)
 
     # test predictions data-type, length and values
     assert isinstance(predictions, (pd.Series, pd.DataFrame))
-    assert len(predictions) == len(boston_target_sr)
+    assert len(predictions) == len(diabetes_target_sr)
 
 
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearndf_cls", argvalues=REGRESSORS_PARTIAL_FIT
 )
 def test_wrapped_partial_fit(
-    sklearndf_cls: Type[RegressorDF],
-    boston_features: pd.DataFrame,
-    boston_target_sr: pd.Series,
-    boston_target_df: pd.DataFrame,
+    sklearndf_cls: type[RegressorDF],
+    diabetes_features: pd.DataFrame,
+    diabetes_target_sr: pd.Series,
+    diabetes_target_df: pd.DataFrame,
 ) -> None:
-
     # noinspection PyArgumentList
     regressor = sklearndf_cls(
         **DEFAULT_REGRESSOR_PARAMETERS.get(f"{sklearndf_cls.__name__}_partial_fit", {})
     )
 
     is_multi_output = isinstance(regressor.native_estimator, MultiOutputRegressor)
-    boston_target = boston_target_df if is_multi_output else boston_target_sr
+    diabetes_target = diabetes_target_df if is_multi_output else diabetes_target_sr
 
-    regressor.partial_fit(boston_features, boston_target)
+    # noinspection PyUnresolvedReferences
+    regressor.partial_fit(diabetes_features, diabetes_target)
