@@ -1,6 +1,7 @@
 import itertools
+from collections.abc import Iterable
 from types import ModuleType
-from typing import Dict, Iterable, List, Optional, Type, TypeVar, Union
+from typing import Optional, TypeVar, Union
 
 import pytest
 import sklearn
@@ -20,8 +21,8 @@ import sklearndf.pipeline
 import sklearndf.regression
 import sklearndf.transformation
 from ..conftest import UNSUPPORTED_SKLEARN_PACKAGES
-from ..sklearndf import find_all_submodules, iterate_classes, sklearn_delegate_classes
-from sklearndf import EstimatorDF, __sklearn_0_23__, __sklearn_version__
+from . import find_all_submodules, iterate_classes, sklearn_delegate_classes
+from sklearndf import EstimatorDF
 
 T = TypeVar("T")
 
@@ -39,21 +40,18 @@ CLASSIFIER_COVERAGE_EXCLUSIONS = {
     *GENERAL_COVERAGE_EXCLUSIONS,
     # Base classes and Mixins not following the convention
     "ForestClassifier",
+    "_IdentityClassifier",
 }
 
-if __sklearn_version__ >= __sklearn_0_23__:
-    added_in_v023 = ("_IdentityClassifier",)
-    CLASSIFIER_COVERAGE_EXCLUSIONS.update(added_in_v023)
 
 REGRESSOR_COVERAGE_EXCLUSIONS = {
     *GENERAL_COVERAGE_EXCLUSIONS,
-    # Base classes and Mixins, not following the convention -->
+    # Base classes and mix-ins
     "ForestRegressor",
-    # <--- Base classes and Mixins, not following the convention
-    # Private classes -->
+    "GeneralizedLinearRegressor",
+    # Private classes
     "_SigmoidCalibration",
-    "_PLS"
-    # <-- Private classes
+    "_PLS",
 }
 
 
@@ -79,7 +77,7 @@ UNSUPPORTED_SKLEARN_CLASSES = {
     sklearn_class.__name__
     for sklearn_class in iterate_classes(
         from_modules=itertools.chain.from_iterable(
-            find_all_submodules(p) for p in UNSUPPORTED_SKLEARN_PACKAGES
+            (p, *find_all_submodules(p)) for p in UNSUPPORTED_SKLEARN_PACKAGES
         ),
         matching=".*",
     )
@@ -88,9 +86,9 @@ UNSUPPORTED_SKLEARN_CLASSES = {
 
 def _find_sklearn_classes_to_cover(
     from_modules: Union[ModuleType, Iterable[ModuleType]],
-    subclass_of: Type[T],
+    subclass_of: type[T],
     excluding: Optional[Union[str, Iterable[str]]] = None,
-) -> List[Type[T]]:
+) -> list[type[T]]:
     return [
         cls
         for cls in iterate_classes(
@@ -100,7 +98,7 @@ def _find_sklearn_classes_to_cover(
     ]
 
 
-def sklearn_classifier_classes() -> List[type]:
+def sklearn_classifier_classes() -> list[type]:
     return _find_sklearn_classes_to_cover(
         from_modules=find_all_submodules(sklearn),
         subclass_of=ClassifierMixin,
@@ -108,7 +106,7 @@ def sklearn_classifier_classes() -> List[type]:
     )
 
 
-def sklearn_regressor_classes() -> List[type]:
+def sklearn_regressor_classes() -> list[type]:
     return _find_sklearn_classes_to_cover(
         from_modules=find_all_submodules(sklearn),
         subclass_of=RegressorMixin,
@@ -116,8 +114,7 @@ def sklearn_regressor_classes() -> List[type]:
     )
 
 
-def sklearn_pipeline_classes() -> List[type]:
-
+def sklearn_pipeline_classes() -> list[type]:
     pipeline_modules = find_all_submodules(sklearn.pipeline)
     pipeline_modules.add(sklearn.pipeline)
 
@@ -128,7 +125,7 @@ def sklearn_pipeline_classes() -> List[type]:
     )
 
 
-def sklearn_transformer_classes() -> List[type]:
+def sklearn_transformer_classes() -> list[type]:
     """Return all classes that are 'just' transformers, not learners or pipelines."""
     transformer_mixin_classes = [
         cls
@@ -151,7 +148,7 @@ def sklearn_transformer_classes() -> List[type]:
     return transformer_classes
 
 
-def sklearn_clusterer_classes() -> List[type]:
+def sklearn_clusterer_classes() -> list[type]:
     return _find_sklearn_classes_to_cover(
         from_modules=find_all_submodules(sklearn),
         subclass_of=ClusterMixin,
@@ -162,19 +159,19 @@ def sklearn_clusterer_classes() -> List[type]:
 def _check_unexpected_sklearn_class(cls: type) -> None:
     f_cls_name = f"{cls.__module__}.{cls.__name__}"
     if cls.__name__ in UNSUPPORTED_SKLEARN_CLASSES:
-        pytest.skip(f"Class '{f_cls_name} is not wrapped but marked as unsupported' ")
+        pytest.skip(f"Class {f_cls_name} is not wrapped but marked as unsupported")
     else:
-        raise ValueError(f"Class '{f_cls_name}' is not wrapped")
+        raise ValueError(f"Class {f_cls_name} is not wrapped")
 
 
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearn_classifier_cls", argvalues=sklearn_classifier_classes()
 )
-def test_classifier_coverage(sklearn_classifier_cls: Type[ClassifierMixin]) -> None:
+def test_classifier_coverage(sklearn_classifier_cls: type[ClassifierMixin]) -> None:
     """Check if each sklearn classifier has a wrapped sklearndf counterpart."""
-    sklearn_classes: Dict[
-        Type[BaseEstimator], Type[EstimatorDF]
-    ] = sklearn_delegate_classes(sklearndf.classification)
+    sklearn_classes: dict[type[BaseEstimator], type[EstimatorDF]] = (
+        sklearn_delegate_classes(sklearndf.classification)
+    )
 
     if sklearn_classifier_cls not in sklearn_classes:
         _check_unexpected_sklearn_class(sklearn_classifier_cls)
@@ -183,11 +180,11 @@ def test_classifier_coverage(sklearn_classifier_cls: Type[ClassifierMixin]) -> N
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearn_regressor_cls", argvalues=sklearn_regressor_classes()
 )
-def test_regressor_coverage(sklearn_regressor_cls: Type[RegressorMixin]) -> None:
+def test_regressor_coverage(sklearn_regressor_cls: type[RegressorMixin]) -> None:
     """Check if each sklearn regressor has a wrapped sklearndf counterpart."""
-    sklearn_classes: Dict[
-        Type[BaseEstimator], Type[EstimatorDF]
-    ] = sklearn_delegate_classes(sklearndf.regression)
+    sklearn_classes: dict[type[BaseEstimator], type[EstimatorDF]] = (
+        sklearn_delegate_classes(sklearndf.regression)
+    )
 
     if sklearn_regressor_cls not in sklearn_classes:
         _check_unexpected_sklearn_class(sklearn_regressor_cls)
@@ -196,12 +193,12 @@ def test_regressor_coverage(sklearn_regressor_cls: Type[RegressorMixin]) -> None
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearn_transformer_cls", argvalues=sklearn_transformer_classes()
 )
-def test_transformer_coverage(sklearn_transformer_cls: Type[TransformerMixin]) -> None:
+def test_transformer_coverage(sklearn_transformer_cls: type[TransformerMixin]) -> None:
     """Check if each sklearn transformer has a wrapped sklearndf counterpart."""
 
-    sklearn_classes: Dict[
-        Type[BaseEstimator], Type[EstimatorDF]
-    ] = sklearn_delegate_classes(sklearndf.transformation)
+    sklearn_classes: dict[type[BaseEstimator], type[EstimatorDF]] = (
+        sklearn_delegate_classes(sklearndf.transformation)
+    )
 
     if sklearn_transformer_cls not in sklearn_classes:
         _check_unexpected_sklearn_class(sklearn_transformer_cls)
@@ -210,7 +207,7 @@ def test_transformer_coverage(sklearn_transformer_cls: Type[TransformerMixin]) -
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearn_pipeline_cls", argvalues=sklearn_pipeline_classes()
 )
-def test_pipeline_coverage(sklearn_pipeline_cls: Type[Pipeline]) -> None:
+def test_pipeline_coverage(sklearn_pipeline_cls: type[Pipeline]) -> None:
     """Check if each sklearn pipeline estimator has
     a wrapped sklearndf counterpart."""
 
@@ -224,11 +221,11 @@ def test_pipeline_coverage(sklearn_pipeline_cls: Type[Pipeline]) -> None:
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearn_clusterer_cls", argvalues=sklearn_clusterer_classes()
 )
-def test_clusterer_coverage(sklearn_clusterer_cls: Type[ClusterMixin]) -> None:
+def test_clusterer_coverage(sklearn_clusterer_cls: type[ClusterMixin]) -> None:
     """Check if each sklearn clusterer has a wrapped sklearndf counterpart."""
-    sklearn_classes: Dict[
-        Type[BaseEstimator], Type[EstimatorDF]
-    ] = sklearn_delegate_classes(sklearndf.clustering)
+    sklearn_classes: dict[type[BaseEstimator], type[EstimatorDF]] = (
+        sklearn_delegate_classes(sklearndf.clustering)
+    )
 
     if sklearn_clusterer_cls not in sklearn_classes:
         _check_unexpected_sklearn_class(sklearn_clusterer_cls)
