@@ -1,10 +1,11 @@
-from typing import Any, List, Type, cast
+from typing import Any, cast
 
 import numpy as np
 import pandas as pd
 import pytest
 import sklearn
 from numpy.testing import assert_array_equal
+from pandas.arrays import SparseArray
 from pandas.testing import assert_frame_equal
 from sklearn.base import BaseEstimator, TransformerMixin, is_classifier, is_regressor
 from sklearn.compose import ColumnTransformer
@@ -21,10 +22,7 @@ from sklearndf import (
     ClassifierDF,
     RegressorDF,
     TransformerDF,
-    __sklearn_0_22__,
-    __sklearn_0_23__,
-    __sklearn_0_24__,
-    __sklearn_1_0__,
+    __sklearn_1_8__,
     __sklearn_version__,
 )
 from sklearndf.classification import RandomForestClassifierDF
@@ -37,6 +35,7 @@ from sklearndf.transformation import (
     NormalizerDF,
     OneHotEncoderDF,
     SelectFromModelDF,
+    SequentialFeatureSelectorDF,
     SimpleImputerDF,
     SparseCoderDF,
     StandardScalerDF,
@@ -52,13 +51,10 @@ TRANSFORMER_EXCLUSIONS = [
     KBinsDiscretizerDF.__name__,
     RFECVDF.__name__,
     RFEDF.__name__,
+    SequentialFeatureSelectorDF.__name__,
     r".*WrapperDF",
 ]
 
-if __sklearn_version__ >= __sklearn_0_24__:
-    from sklearndf.transformation import SequentialFeatureSelectorDF
-
-    TRANSFORMER_EXCLUSIONS.append(SequentialFeatureSelectorDF.__name__)
 
 # noinspection PyTypeChecker
 TRANSFORMERS_TO_TEST = iterate_classes(
@@ -73,14 +69,10 @@ def test_transformer_count() -> None:
     n = len(TRANSFORMERS_TO_TEST)
 
     print(f"Testing {n} transformers.")
-    if __sklearn_version__ < __sklearn_0_22__:
-        assert n == 53
-    elif __sklearn_version__ < __sklearn_0_24__:
-        assert n == 54
-    elif __sklearn_version__ < __sklearn_1_0__:
-        assert n == 55
+    if __sklearn_version__ < __sklearn_1_8__:
+        assert n == 61
     else:
-        assert n == 56
+        pytest.fail(f"unexpected sklearn version: {__sklearn_version__}")
 
 
 @pytest.fixture  # type: ignore
@@ -98,7 +90,7 @@ def test_data() -> pd.DataFrame:
 @pytest.mark.parametrize(  # type: ignore
     argnames="sklearndf_cls", argvalues=TRANSFORMERS_TO_TEST
 )
-def test_wrapped_constructor(sklearndf_cls: Type[TransformerDF]) -> None:
+def test_wrapped_constructor(sklearndf_cls: type[TransformerDF]) -> None:
     transformer_df: TransformerDF = sklearndf_cls()
 
     if isinstance(transformer_df, RegressorDF):
@@ -120,10 +112,6 @@ def test_wrapped_constructor(sklearndf_cls: Type[TransformerDF]) -> None:
 def test_special_wrapped_constructors() -> None:
     rf = RandomForestClassifierDF()
 
-    with pytest.raises(NotImplementedError):
-        OneHotEncoderDF()
-    OneHotEncoderDF(sparse=False)
-
     SelectFromModelDF(estimator=rf)
 
     SparseCoderDF(dictionary=np.array([]))
@@ -138,10 +126,7 @@ def test_special_wrapped_constructors() -> None:
 
     RFEDF(estimator=rf)
 
-    if __sklearn_version__ >= __sklearn_0_24__:
-        from sklearndf.transformation import SequentialFeatureSelectorDF
-
-        SequentialFeatureSelectorDF(estimator=rf)
+    SequentialFeatureSelectorDF(estimator=rf)
 
 
 @pytest.mark.parametrize(  # type: ignore
@@ -152,7 +137,7 @@ def test_special_wrapped_constructors() -> None:
     ),
 )
 def test_fit_transform(
-    sklearn_cls: Type[BaseEstimator], test_data: pd.DataFrame
+    sklearn_cls: type[BaseEstimator], test_data: pd.DataFrame
 ) -> None:
     # we only need the numerical column of the test data
     test_data = test_data.select_dtypes(include=float)
@@ -192,44 +177,43 @@ def test_fit_transform(
     # test inverse transform
 
     inverse_transformed_df = transformer_df.inverse_transform(X=transformed_df)
-    assert_frame_equal(
-        inverse_transformed_df, test_data.rename_axis(columns="feature_in")
-    )
+    assert_frame_equal(inverse_transformed_df, test_data.rename_axis(columns="feature"))
 
     # test feature names in and out
-    if __sklearn_version__ >= __sklearn_1_0__:
-        # noinspection PyUnresolvedReferences
-        assert_array_equal(
-            transformer_df.feature_names_in_.values,
-            transformer_native.feature_names_in_,
-        )
-        # noinspection PyUnresolvedReferences
-        assert_array_equal(
-            transformer_df.feature_names_out_.values,
-            transformer_native.get_feature_names_out(),
-        )
-        # noinspection PyUnresolvedReferences
-        assert_array_equal(
-            transformer_df.feature_names_original_.index.values,
-            transformer_native.get_feature_names_out(),
-        )
+
+    # noinspection PyUnresolvedReferences
+    assert_array_equal(
+        transformer_df.feature_names_in_.values,
+        transformer_native.feature_names_in_,
+    )
+    # noinspection PyUnresolvedReferences
+    assert_array_equal(
+        transformer_df.feature_names_out_.values,
+        transformer_native.get_feature_names_out(),
+    )
+    # noinspection PyUnresolvedReferences
+    assert_array_equal(
+        transformer_df.feature_names_original_.index.values,
+        transformer_native.get_feature_names_out(),
+    )
 
 
 def test_column_transformer(test_data: pd.DataFrame) -> None:
-    numeric_columns: List[str] = test_data.select_dtypes(include=float).columns.tolist()
+    numeric_columns: list[str] = test_data.select_dtypes(include=float).columns.tolist()
     assert numeric_columns == ["c0", "c2"]
 
     # noinspection PyShadowingNames
     def _test_transformer(
+        *,
         remainder: str,
-        names_in: List[str],
-        names_original: List[str],
-        names_out: List[str],
+        names_in: list[str],
+        names_original: list[str],
+        names_out: list[str],
         **transformer_args: Any,
     ) -> None:
-        feature_names_out_expected = pd.Index(names_out, name="feature_out")
+        feature_names_out_expected = pd.Index(names_out, name="feature")
         data = test_data.loc[:, names_in]
-        feature_names_in_expected = data.columns.rename("feature_in")
+        feature_names_in_expected = data.columns.rename("feature")
 
         # test fit-transform in connection with ColumnTransformer(DF)
         tx_df = StandardScalerDF()
@@ -252,80 +236,80 @@ def test_column_transformer(test_data: pd.DataFrame) -> None:
             remainder=remainder,
             **transformer_args,
         )
-        transformed_native = col_tx_native.fit_transform(X=data)
-
-        assert_frame_equal(
-            transformed_df,
-            pd.DataFrame(transformed_native, columns=feature_names_out_expected),
+        transformed_native = pd.DataFrame(
+            col_tx_native.fit_transform(X=data), columns=feature_names_out_expected
         )
 
+        assert_frame_equal(transformed_df, transformed_native, check_dtype=False)
+
         assert col_tx_df.feature_names_in_.equals(feature_names_in_expected)
         assert col_tx_df.feature_names_out_.equals(feature_names_out_expected)
         assert col_tx_df.feature_names_original_.equals(
             pd.Series(names_original, index=feature_names_out_expected)
         )
 
-        if __sklearn_version__ >= __sklearn_1_0__:
-            # noinspection PyUnresolvedReferences
-            assert_array_equal(
-                col_tx_df.feature_names_in_.values, col_tx_native.feature_names_in_
-            )
+        # noinspection PyUnresolvedReferences
+        assert_array_equal(
+            col_tx_df.feature_names_in_.values, col_tx_native.feature_names_in_
+        )
 
-    if __sklearn_version__ < __sklearn_1_0__:
-        _test_transformer(
-            remainder="passthrough",
-            names_in=["c0", "c2", "c3", "c1"],
-            names_original=["c0", "c2", "c3", "c1"],
-            names_out=["c0", "c2", "c3", "c1"],
-        )
-        _test_transformer(
-            remainder="drop",
-            names_in=["c0", "c2", "c3"],
-            names_original=["c0", "c2", "c3"],
-            names_out=["c0", "c2", "c3"],
-        )
-        _test_transformer(
-            remainder="drop",
-            names_in=["c0", "c2", "c3", "c1"],
-            names_original=["c0", "c2", "c3"],
-            names_out=["c0", "c2", "c3"],
-        )
-    else:
-        # As of scikit-learn 1.0, column transformers have a new boolean parameter
-        # verbose_feature_names_out. We test once with verbosity disabled, once
-        # with verbosity enabled, and once with the default (enabled).
-        _test_transformer(
-            remainder="passthrough",
-            names_in=["c0", "c2", "c3", "c1"],
-            names_original=["c0", "c2", "c3", "c1"],
-            names_out=["tx__c0", "tx__c2", "keep__c3", "remainder__c1"],
-        )
-        _test_transformer(
-            remainder="drop",
-            names_in=["c0", "c2", "c3"],
-            names_original=["c0", "c2", "c3"],
-            names_out=["tx__c0", "tx__c2", "keep__c3"],
-        )
-        _test_transformer(
-            remainder="drop",
-            names_in=["c0", "c2", "c3", "c1"],
-            names_original=["c0", "c2", "c3"],
-            names_out=["tx__c0", "tx__c2", "keep__c3"],
-        )
-        _test_transformer(
-            remainder="drop",
-            names_in=["c0", "c2", "c3", "c1"],
-            names_original=["c0", "c2", "c3"],
-            names_out=["tx__c0", "tx__c2", "keep__c3"],
-            verbose_feature_names_out=True,
-        )
-        _test_transformer(
-            remainder="drop",
-            names_in=["c0", "c2", "c3", "c1"],
-            names_original=["c0", "c2", "c3"],
-            names_out=["c0", "c2", "c3"],
-            verbose_feature_names_out=False,
-        )
+    # As of scikit-learn 1.0, column transformers have a new boolean parameter
+    # verbose_feature_names_out. We test once with verbosity disabled, once
+    # with verbosity enabled, and once with the default (enabled).
+    _test_transformer(
+        remainder="passthrough",
+        names_in=["c0", "c2", "c3", "c1"],
+        names_original=["c0", "c2", "c3", "c1"],
+        names_out=["tx__c0", "tx__c2", "keep__c3", "remainder__c1"],
+    )
+    _test_transformer(
+        remainder="drop",
+        names_in=["c0", "c2", "c3"],
+        names_original=["c0", "c2", "c3"],
+        names_out=["tx__c0", "tx__c2", "keep__c3"],
+    )
+    _test_transformer(
+        remainder="drop",
+        names_in=["c0", "c2", "c3", "c1"],
+        names_original=["c0", "c2", "c3"],
+        names_out=["tx__c0", "tx__c2", "keep__c3"],
+    )
+    _test_transformer(
+        remainder="drop",
+        names_in=["c0", "c2", "c3", "c1"],
+        names_original=["c0", "c2", "c3"],
+        names_out=["tx__c0", "tx__c2", "keep__c3"],
+        verbose_feature_names_out=True,
+    )
+    _test_transformer(
+        remainder="drop",
+        names_in=["c0", "c2", "c3", "c1"],
+        names_original=["c0", "c2", "c3"],
+        names_out=["c0", "c2", "c3"],
+        verbose_feature_names_out=False,
+    )
+
+
+def test_column_transformer_with_unmatched_transformation(
+    test_data: pd.DataFrame,
+) -> None:
+    # Test that a ColumnTransformerDF with a transformer that does not match any
+    # columns does not raise an error.
+    numeric_data = test_data.select_dtypes(include=float)
+    numeric_columns: list[str] = numeric_data.columns.tolist()
+    assert numeric_columns == ["c0", "c2"]
+    # noinspection PyShadowingNames
+    tx = ColumnTransformerDF(
+        transformers=[
+            ("tx", StandardScalerDF(), []),
+            ("rest", "passthrough", numeric_columns),
+        ]
+    )
+    transformed_df = tx.fit_transform(X=test_data)
+    # check that the transformed DataFrame is the same as the input DataFrame
+    assert_frame_equal(
+        transformed_df, numeric_data.add_prefix("rest__").rename_axis("feature", axis=1)
+    )
 
 
 def test_normalizer_df() -> None:
@@ -337,7 +321,7 @@ def test_normalizer_df() -> None:
 
     transformed_non_df = pd.DataFrame(
         non_df_normalizer.fit_transform(X=x),
-        columns=pd.Index(["a", "b", "c", "d"], name="feature_out"),
+        columns=pd.Index(["a", "b", "c", "d"], name="feature"),
     )
 
     # test fit_trannsform
@@ -363,7 +347,7 @@ def test_simple_imputer_df() -> None:
     x = np.array(
         [[4.0, 1.0, 2.0, np.nan], [1.0, np.nan, 9.0, 3.0], [np.nan, np.nan, 5.0, 1.0]]
     )
-    x_df = pd.DataFrame(x, columns=pd.Index(["a", "b", "c", "d"], name="feature_in"))
+    x_df = pd.DataFrame(x, columns=pd.Index(["a", "b", "c", "d"], name="feature"))
 
     imputer_native = SimpleImputer(add_indicator=True)
     imputer_df = SimpleImputerDF(add_indicator=True)
@@ -372,8 +356,16 @@ def test_simple_imputer_df() -> None:
     transformed_df_expected = pd.DataFrame(
         transformed_native,
         columns=pd.Index(
-            ["a", "b", "c", "d", "a__missing", "b__missing", "d__missing"],
-            name="feature_out",
+            [
+                "a",
+                "b",
+                "c",
+                "d",
+                "missingindicator_a",
+                "missingindicator_b",
+                "missingindicator_d",
+            ],
+            name="feature",
         ),
     )
 
@@ -390,16 +382,18 @@ def test_simple_imputer_df() -> None:
 
     # test inverse transform
 
-    if __sklearn_version__ >= __sklearn_0_24__:
-        inverse_transformed_df = imputer_df.inverse_transform(X=transformed_df)
-        assert_frame_equal(inverse_transformed_df, x_df)
+    inverse_transformed_df = imputer_df.inverse_transform(X=transformed_df)
+    assert_frame_equal(inverse_transformed_df, x_df)
 
     # test feature names in and out
-    if __sklearn_version__ >= __sklearn_1_0__:
-        # noinspection PyUnresolvedReferences
-        assert_array_equal(
-            imputer_df.feature_names_in_.values, imputer_native.feature_names_in_
-        )
+    assert_array_equal(
+        imputer_df.feature_names_in_.values, imputer_native.feature_names_in_
+    )
+
+    assert_array_equal(
+        imputer_df.feature_names_out_.values,
+        imputer_native.get_feature_names_out(),
+    )
 
 
 @pytest.fixture  # type: ignore
@@ -414,19 +408,26 @@ def df_outlier() -> pd.DataFrame:
     )
 
 
-def test_one_hot_encoding() -> None:
-    test_data_categorical = pd.DataFrame(
-        data=[
-            ["yes", "red", "child"],
-            ["yes", "blue", "father"],
-            ["no", "green", "mother"],
-        ],
-        columns=["a", "b", "c"],
-    )
+@pytest.mark.parametrize(  # type: ignore
+    argnames="sparse_output", argvalues=[True, False]
+)
+def test_one_hot_encoding(
+    test_data_categorical: pd.DataFrame, sparse_output: bool
+) -> None:
+    def _make_frame(data: dict[str, list[float]]) -> pd.DataFrame:
+        if sparse_output:
+            df = pd.DataFrame(
+                data={k: SparseArray(v, fill_value=0) for k, v in data.items()}
+            )
+        else:
+            df = pd.DataFrame(data=data)
+        return df.rename_axis(columns="feature")
 
     assert_frame_equal(
-        OneHotEncoderDF(drop=None, sparse=False).fit_transform(test_data_categorical),
-        pd.DataFrame(
+        OneHotEncoderDF(drop=None, sparse_output=sparse_output).fit_transform(
+            test_data_categorical
+        ),
+        _make_frame(
             {
                 "a_no": [0.0, 0.0, 1.0],
                 "a_yes": [1.0, 1.0, 0.0],
@@ -437,32 +438,14 @@ def test_one_hot_encoding() -> None:
                 "c_father": [0.0, 1.0, 0.0],
                 "c_mother": [0.0, 0.0, 1.0],
             }
-        ).rename_axis(columns="feature_out"),
+        ),
     )
 
-    if __sklearn_version__ >= __sklearn_0_23__:
-        assert_frame_equal(
-            OneHotEncoderDF(drop="if_binary", sparse=False).fit_transform(
-                test_data_categorical
-            ),
-            pd.DataFrame(
-                {
-                    "a_yes": [1.0, 1.0, 0.0],
-                    "b_blue": [0.0, 1.0, 0.0],
-                    "b_green": [0.0, 0.0, 1.0],
-                    "b_red": [1.0, 0.0, 0.0],
-                    "c_child": [1.0, 0.0, 0.0],
-                    "c_father": [0.0, 1.0, 0.0],
-                    "c_mother": [0.0, 0.0, 1.0],
-                }
-            ).rename_axis(columns="feature_out"),
-        )
-
     assert_frame_equal(
-        OneHotEncoderDF(drop="first", sparse=False).fit_transform(
+        OneHotEncoderDF(drop="first", sparse_output=sparse_output).fit_transform(
             test_data_categorical
         ),
-        pd.DataFrame(
+        _make_frame(
             {
                 "a_yes": [1.0, 1.0, 0.0],
                 "b_green": [0.0, 0.0, 1.0],
@@ -470,14 +453,14 @@ def test_one_hot_encoding() -> None:
                 "c_father": [0.0, 1.0, 0.0],
                 "c_mother": [0.0, 0.0, 1.0],
             }
-        ).rename_axis(columns="feature_out"),
+        ),
     )
 
     assert_frame_equal(
-        OneHotEncoderDF(drop=["yes", "red", "mother"], sparse=False).fit_transform(
-            test_data_categorical
-        ),
-        pd.DataFrame(
+        OneHotEncoderDF(
+            drop=["yes", "red", "mother"], sparse_output=sparse_output
+        ).fit_transform(test_data_categorical),
+        _make_frame(
             {
                 "a_no": [0.0, 0.0, 1.0],
                 "b_blue": [0.0, 1.0, 0.0],
@@ -485,5 +468,113 @@ def test_one_hot_encoding() -> None:
                 "c_child": [1.0, 0.0, 0.0],
                 "c_father": [0.0, 1.0, 0.0],
             }
-        ).rename_axis(columns="feature_out"),
+        ),
+    )
+
+    assert_frame_equal(
+        OneHotEncoderDF(drop="if_binary", sparse_output=sparse_output).fit_transform(
+            test_data_categorical
+        ),
+        _make_frame(
+            {
+                "a_yes": [1.0, 1.0, 0.0],
+                "b_blue": [0.0, 1.0, 0.0],
+                "b_green": [0.0, 0.0, 1.0],
+                "b_red": [1.0, 0.0, 0.0],
+                "c_child": [1.0, 0.0, 0.0],
+                "c_father": [0.0, 1.0, 0.0],
+                "c_mother": [0.0, 0.0, 1.0],
+            }
+        ),
+    )
+
+    assert_frame_equal(
+        OneHotEncoderDF(min_frequency=2, sparse_output=sparse_output).fit_transform(
+            test_data_categorical
+        ),
+        _make_frame(
+            {
+                "a_yes": [1.0, 1.0, 0.0],
+                "a_infrequent_sklearn": [0.0, 0.0, 1.0],
+                "b_infrequent_sklearn": [1.0, 1.0, 1.0],
+                "c_infrequent_sklearn": [1.0, 1.0, 1.0],
+            }
+        ),
+    )
+
+    assert_frame_equal(
+        OneHotEncoderDF(max_categories=2, sparse_output=sparse_output).fit_transform(
+            test_data_categorical
+        ),
+        _make_frame(
+            {
+                "a_yes": [1.0, 1.0, 0.0],
+                "a_infrequent_sklearn": [0.0, 0.0, 1.0],
+                "b_red": [1.0, 0.0, 0.0],
+                "b_infrequent_sklearn": [0.0, 1.0, 1.0],
+                "c_mother": [0.0, 0.0, 1.0],
+                "c_infrequent_sklearn": [1.0, 1.0, 0.0],
+            }
+        ),
+    )
+
+    assert_frame_equal(
+        OneHotEncoderDF(max_categories=10, sparse_output=sparse_output).fit_transform(
+            test_data_categorical
+        ),
+        _make_frame(
+            {
+                "a_no": [0.0, 0.0, 1.0],
+                "a_yes": [1.0, 1.0, 0.0],
+                "b_blue": [0.0, 1.0, 0.0],
+                "b_green": [0.0, 0.0, 1.0],
+                "b_red": [1.0, 0.0, 0.0],
+                "c_child": [1.0, 0.0, 0.0],
+                "c_father": [0.0, 1.0, 0.0],
+                "c_mother": [0.0, 0.0, 1.0],
+            }
+        ),
+    )
+
+    assert_frame_equal(
+        OneHotEncoderDF(
+            drop="if_binary", max_categories=10, sparse_output=sparse_output
+        ).fit_transform(test_data_categorical),
+        _make_frame(
+            {
+                "a_yes": [1.0, 1.0, 0.0],
+                "b_blue": [0.0, 1.0, 0.0],
+                "b_green": [0.0, 0.0, 1.0],
+                "b_red": [1.0, 0.0, 0.0],
+                "c_child": [1.0, 0.0, 0.0],
+                "c_father": [0.0, 1.0, 0.0],
+                "c_mother": [0.0, 0.0, 1.0],
+            }
+        ),
+    )
+
+    assert_frame_equal(
+        OneHotEncoderDF(
+            drop="first", max_categories=2, sparse_output=sparse_output
+        ).fit_transform(test_data_categorical),
+        _make_frame(
+            {
+                "a_infrequent_sklearn": [0.0, 0.0, 1.0],
+                "b_infrequent_sklearn": [0.0, 1.0, 1.0],
+                "c_infrequent_sklearn": [1.0, 1.0, 0.0],
+            }
+        ),
+    )
+
+    assert_frame_equal(
+        OneHotEncoderDF(
+            drop="if_binary", max_categories=2, sparse_output=sparse_output
+        ).fit_transform(test_data_categorical),
+        _make_frame(
+            {
+                "a_infrequent_sklearn": [0.0, 0.0, 1.0],
+                "b_infrequent_sklearn": [0.0, 1.0, 1.0],
+                "c_infrequent_sklearn": [1.0, 1.0, 0.0],
+            }
+        ),
     )
